#include <cassert>
#include <fstream>
#include <vector>
#include <iostream>
#include <iomanip>

#include "waveform.h"

// import from wav.cpp
void WriteWav(std::string filename, std::vector<float> const& samples, int sampleRate);

namespace
{
typedef std::vector<float> AnalogBuffer;

class TestFailure
{
};

void workFilter(Filter& pFilter, size_t iNumSamples, std::vector<float>& buffer)
{
  for(int i = 0; i < iNumSamples; ++i)
  {
    auto const sample = pFilter.Work(0);

    if(sample != sample)
    {
      std::cout << "NaN occured!" << std::endl;
      throw TestFailure();
    }
    else if(sample > 1.0f)
    {
      std::cout << "sample=" << sample << " (too high)" << std::endl;
      throw TestFailure();
    }
    else if(sample < -1.0f)
    {
      std::cout << "sample=" << sample << " (too low)" << std::endl;
      throw TestFailure();
    }

    buffer.push_back(sample);
  }
}
}

template<WaveForm::SignalType shape>
void testWaveForm()
{
  int const SAMPLING_RATE = 44100;
  WaveForm w(shape);
  w.SetSampleRate(SAMPLING_RATE);

  std::vector<float> buffer;

  for(int i = 0; i < 10; ++i)
  {
    w.SetFrequency(110.0);
    workFilter(w, 4096, buffer);

    w.SetFrequency(110.0 * (i + 1) * 3);
    workFilter(w, 1024, buffer);
  }

  WriteWav("/tmp/yo.wav", buffer, SAMPLING_RATE);
}

typedef void testerFunction ();

void run_test(std::string testName, testerFunction* pTest)
{
  std::cout << "****************************************" << std::endl;
  std::cout << "Test: " << testName << std::endl;
  bool bSuccess = false;
  try
  {
    (*pTest)();
    bSuccess = true;
  }
  catch(...)
  {
    std::cout << "Uncaught exception during test" << std::endl;
  }
  std::cout << "Terminated: " << (bSuccess ? "Success" : "Failure") << std::endl;
}

int main()
{
  run_test("WaveForm(Sawtooth)", &testWaveForm<WaveForm::SAWTOOTH> );
  run_test("WaveForm(Pulse)", &testWaveForm<WaveForm::PULSE> );
  run_test("WaveForm(Square)", &testWaveForm<WaveForm::SQUARE> );
  run_test("WaveForm(Triangle)", &testWaveForm<WaveForm::TRIANGLE> );
  return 0;
}

