#pragma once

typedef struct Vector
{
  void** body;
  int len;
  int nalloc;
}Vector;

int vec_len(Vector* vec);
void* vec_get(Vector* vec, int index);

typedef struct
{
  struct Map* map;
  Vector* key;
}Dict;

typedef struct Type Type;

typedef struct Type
{
  int kind;
  int size;
  int align;
  bool usig; // true if unsigned
  bool isstatic;
  // pointer or array
  Type* ptr;
  // array length
  int len;
  // struct
  Dict* fields;
  int offset;
  bool is_struct; // true if struct, false if union
  // bitfield
  int bitoff;
  int bitsize;
  // function
  Type* rettype;
  Vector* params;
  bool hasva;
  bool oldstyle;
}Type;

typedef struct
{
  char* file;
  int line;
}SourceLoc;

enum
{
  KIND_VOID,
  KIND_BOOL,
  KIND_CHAR,
  KIND_SHORT,
  KIND_INT,
  KIND_LONG,
  KIND_LLONG,
  KIND_FLOAT,
  KIND_DOUBLE,
  KIND_LDOUBLE,
  KIND_ARRAY,
  KIND_ENUM,
  KIND_PTR,
  KIND_STRUCT,
  KIND_FUNC,
  // used only in parser
  KIND_STUB,
};

enum
{
  AST_LITERAL = 256,
  AST_LVAR,
  AST_GVAR,
  AST_TYPEDEF,
  AST_FUNCALL,
  AST_FUNCPTR_CALL,
  AST_FUNCDESG,
  AST_FUNC,
  AST_DECL,
  AST_INIT,
  AST_CONV,
  AST_ADDR,
  AST_DEREF,
  AST_IF,
  AST_TERNARY,
  AST_DEFAULT,
  AST_RETURN,
  AST_COMPOUND_STMT,
  AST_STRUCT_REF,
  AST_GOTO,
  AST_COMPUTED_GOTO,
  AST_LABEL,
  OP_SIZEOF,
  OP_CAST,
  OP_SHR,
  OP_SHL,
  OP_A_SHR,
  OP_A_SHL,
  OP_PRE_INC,
  OP_PRE_DEC,
  OP_POST_INC,
  OP_POST_DEC,
  OP_LABEL_ADDR,
#define op(name, _) name,
#define keyword(name, x, y) name,
#include "keyword.inc"
#undef keyword
#undef op
};

typedef struct Node Node;

typedef struct Node
{
  int kind;
  Type* ty;
  SourceLoc* sourceLoc;
  void* user;
  union
  {
    // Char, int, or long
    long ival;
    // Float or double
    struct
    {
      double fval;
      char* flabel;
    };
    // String
    struct
    {
      char* sval;
      char* slabel;
    };
    // Local/global variable
    struct
    {
      char* varname;
      // local
      int loff;
      Vector* lvarinit;
      // global
      char* glabel;
    };
    // Binary operator
    struct
    {
      Node* left;
      Node* right;
    };
    // Unary operator
    struct
    {
      Node* operand;
    };
    // Function call or function declaration
    struct
    {
      char* fname;
      // Function call
      Vector* args;
      Type* ftype;
      // Function pointer or function designator
      Node* fptr;
      // Function declaration
      Vector* params;
      Vector* localvars;
      Node* body;
    };
    // Declaration
    struct
    {
      Node* declvar;
      Vector* declinit;
    };
    // Initializer
    struct
    {
      Node* initval;
      int initoff;
      Type* totype;
    };
    // If statement or ternary operator
    struct
    {
      Node* cond;
      Node* then;
      Node* els;
    };
    // Goto and label
    struct
    {
      char* label;
      char* newlabel;
    };
    // Return statement
    Node* retval;
    // Compound statement
    Vector* stmts;
    // Struct reference
    struct
    {
      Node* struc;
      char* field;
      Type* fieldtype;
    };
  };
}Node;

char* node2s(Node* node);

extern Type* type_void;
extern Type* type_bool;
extern Type* type_char;
extern Type* type_short;
extern Type* type_int;
extern Type* type_long;
extern Type* type_llong;
extern Type* type_uchar;
extern Type* type_ushort;
extern Type* type_uint;
extern Type* type_ulong;
extern Type* type_ullong;
extern Type* type_float;
extern Type* type_double;
extern Type* type_ldouble;

