// Delay definition.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#include <cassert>
#include "lib_utils/math_tools.h"
#include "delay.h"

Delay::Delay()
{
  Initialize();
}

void Delay::Initialize()
{
  m_nPos = 0;

  for(auto& val : buffer)
    val = 0;

  m_nDelayLength = MAX_DELAY;
  m_nFeedback = 0.60f;
}

float Delay::Work(float input)
{
  auto delayed = buffer[m_nPos];
  auto output = input + delayed;

  output = Clamp(output, -1.0f, +1.0f);

  buffer[m_nPos] = input + delayed * m_nFeedback;

  if(m_nDelayLength > 0)
    m_nPos = (m_nPos + 1) % m_nDelayLength;

  return output;
}

void Delay::SetParams(float delay, float feedback)
{
  m_nDelayLength = int(delay * MAX_DELAY);
  m_nFeedback = feedback;
}

void Delay::SetTime(int numSamples)
{
  assert(numSamples >= 0 && numSamples < MAX_DELAY);
  m_nDelayLength = numSamples;
}

void Delay::SetFeedback(float feedback)
{
  m_nFeedback = feedback;
}

