// audio synthesizer engine.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#pragma once

#include "voice_dispatcher.h"
#include "delay.h"
#include "phaser.h"
#include "voice.h"
#include "lfo.h"
#include "lib_utils/fraction.h"

Osc::SHAPE ParamToOscShape(float fValue);
Lfo::SHAPE ParamToShape(float fValue);
Fraction ToClock(float fValue);
bool ParamToBool(float fValue);

struct OscParams
{
  float Volume;
  float Shape;
  float Detune;
  float Pitch;
  float PulseWidth;
};

struct EnvParams
{
  float Attack;
  float Decay;
  float Sustain;
  float Release;
  float ToVolume;
  float ToFilter;
  float ToPitch;
};

struct FilterParams
{
  float Cutoff;
  float Resonance;
  float Pan;
};

struct LfoParams
{
  float Shape;
  float Rate;
  float Clock;
  float ToFilter;
  float ToVolume;
  float ToPitch;
};

struct DelayParams
{
  float send;
  float time;
  float feedback;
  float clock;
};

struct PhaserParams
{
  float Send;
  float Freq;
  float Depth;
  float Feedback;
};

struct ParamValues
{
  float m_Volume;
  float m_Mono;
  float m_Portamento;

  OscParams osc1;
  OscParams osc2;

  float m_OscVolume;
  float m_OscRingMod;
  float m_OscBalance;

  EnvParams env1;
  EnvParams env2;

  FilterParams filter1;
  FilterParams filter2;

  float m_FilterBalance;
  float m_FilterPregain;

  LfoParams lfo1;
  LfoParams lfo2;

  float m_VelocityToFilter;
  float m_VelocityToVolume;
  float m_VelocityToPitch;

  float m_ModToFilter;
  float m_ModToVolume;
  float m_ModToPitch;
  float m_ModToLfo1ToPitch;

  // effects
  DelayParams delay;
  PhaserParams phaser;
};

class EchidnaSynth
{
public:
  EchidnaSynth();

  void SetSampleRate(float fSampleRate);

  void ProcessAudio(float** pOutput, long iNumSamples);

  void OnTempoChange(float fTempo);
  void OnNoteOn(int iNote, int iVelocity);
  void OnNoteOff(int iNote, int iVelocity);
  void OnPitchBend(float fBend);
  void OnControlChange(int iControl, int iValue);

  ParamValues m_Values;

private:
  static const int MAX_VOICES = 16;

  Voice m_Voices[MAX_VOICES];
  float m_fSampleRate;
  float m_fTempo;
  float m_fModulationWheel;
  int m_iRemainingSamplesInChunk;

  void UpdateDelay(int index);
  void UpdateLfos(int index, float lfoShape, float lfoClock, float lfoRate);
  void NewChunk();
  void GenerateAudio(AudioBuffer buffer);

  bool m_bKeys[128];
  Delay m_Delay[2];
  Lfo m_Lfos[6];
  Phaser m_Phaser[2];
  VoiceDispatcher m_Dispatcher;
};

