// ADSR envelope class.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#include "filter.h"

class Envelope
{
public:
  Envelope();

  void SetADSR(float fAttack, float fDecay, float fSustain, float fRelease);

  virtual void EnvAttack();
  virtual void EnvRelease();

  void Initialize();
  float Work();
  float Value() const;

  bool IsDead() const;

private:
  float m_fAttack; // attack time, in s
  float m_fAttackIncrement; // pre-computed
  float m_fDecay; // decay time, in s
  float m_fDecayIncrement; // pre-computed
  float m_fSustain; // sustain level, in sample units (0..1)
  float m_fRelease; // release time, in s
  float m_fReleaseIncrement; // pre-computed

  float m_fCurrLevel;

  enum STATE
  {
    ATTACKING,
    DECAYING,
    SUSTAINING,
    RELEASING,
    DEAD,
  };

  bool m_bHolding;
  STATE m_iState;

  /**
   * @brief Position inside the state. Always between 0 and 1. When it has
   * reached 1, it means we should go to the next state (e.g attack has finised)
   */
  float m_fPos;
};

