// Lowpass filter definition.
/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

class Filter
{
public:
  virtual ~Filter() {}

  virtual float Work(float sample) = 0;

protected:
  bool m_bParamsChanged;
};

class Lowpass : public Filter
{
public:
  Lowpass();
  void Initialize();
  float Work(float sample);

  // parameters are passed as percentage of range
  void SetParams(float targetC100, float targetR100);

private:
  float history[4];
  float cutoff;
  float resonance;
  float targetCutoff;
  float targetResonance;
  float minCutoff, maxCutoff;
  float minResonance, maxResonance;
  float cutoffPow, resonancePow;
  float inertia; // ratio of approach to target
  float lastTargetC100, lastTargetR100;
  float coef[5];
};

class Highpass : public Filter
{
public:
  // parameters are passed as percentage of range
  void SetParams(float targetC100, float targetR100);

  virtual float Work(float sample);

private:
  Lowpass m_Lowpass;
};

