/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#include "lfo.h"
#include "lib_utils/math_tools.h"
#include <cmath>

float Lfo::Work()
{
  auto const PERIOD = 2.0 * MY_PI;
  auto SAMPLE_RATE = 44100.0;
  auto dt = 1.0 / SAMPLE_RATE;
  m_fPhase += PERIOD * m_fRate * dt;

  while(m_fPhase >= PERIOD)
    m_fPhase -= PERIOD;
  switch(m_eShape)
  {
  case SINE:
    m_fCurrValue = std::sin(m_fPhase);
    break;
  case TRIANGLE:
    m_fCurrValue = (m_fPhase < PERIOD / 2 ? m_fPhase : PERIOD - m_fPhase) / (PERIOD / 2.0);
    m_fCurrValue = (2 * m_fCurrValue) - 1.0; // set range to [-1, 1]
    break;
  case SAWTOOTH:
    m_fCurrValue = (PERIOD - m_fPhase) / PERIOD;
    m_fCurrValue = (2 * m_fCurrValue) - 1.0; // set range to [-1, 1]
    break;
  case SQUARE:
    m_fCurrValue = (m_fPhase >= PERIOD / 2 ? 1.0 : -1.0);
    break;
  }

  return m_fCurrValue;
}

float Lfo::GetValue() const
{
  return m_fCurrValue;
}

