/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#include <algorithm>
#include "filter.h"

class Lfo
{
public:
  enum SHAPE
  {
    SINE,
    TRIANGLE,
    SAWTOOTH,
    SQUARE,
  };

  static auto const NUM_SHAPES = SQUARE;

  float Work();

  float GetValue() const;

  void SetRate(float fRate)
  {
    m_fRate = std::max(fRate, 0.0f);
  }

  void SetShape(SHAPE shape)
  {
    m_eShape = shape;
  }

private:
  SHAPE m_eShape = SINE;
  float m_fPhase = 0;
  float m_fRate = 0.1;
  float m_fCurrValue = 0;
};

inline char const* ToString(Lfo::SHAPE eShape)
{
  switch(eShape)
  {
  case Lfo::SINE: return "sine";
  case Lfo::TRIANGLE: return "triangle";
  case Lfo::SAWTOOTH: return "saw";
  case Lfo::SQUARE: return "square";
  }

  return "?";
}

