// Oscillator stuff : waveform generation, and phase management.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#include "osc.h"

#include <cassert>

void Osc::SetFrequency(float fFreq)
{
  assert(fFreq <= 0.5f);
  assert(fFreq >= 0.0f);
  m_pCurrWaveForm->SetFrequency(fFreq);
}

void Osc::Sync()
{
  m_pCurrWaveForm->Sync();
}

void Osc::SetWaveform(SHAPE eShape)
{
  switch(eShape)
  {
  case SHAPE_SINE:
    m_pCurrWaveForm = &m_Sine;
    break;
  case SHAPE_TRIANGLE:
    m_pCurrWaveForm = &m_Triangle;
    break;
  case SHAPE_PULSE:
    m_pCurrWaveForm = &m_Pulse;
    break;
  case SHAPE_SAW:
    m_pCurrWaveForm = &m_Sawtooth;
    break;
  case SHAPE_NOISE:
    m_pCurrWaveForm = &m_Noise;
    break;
  }
}

void Osc::SetPulseWidth(float fPulseWidth)
{
  m_Pulse.SetDutyCycle(fPulseWidth);
}

float Osc::Work()
{
  return m_pCurrWaveForm->Work();
}

