// Oscillator stuff : waveform generation, and phase management.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#include "filter.h"
#include "waveform.h"

class Osc
{
public:
  enum SHAPE
  {
    SHAPE_SINE,
    SHAPE_TRIANGLE,
    SHAPE_PULSE,
    SHAPE_SAW,
    SHAPE_NOISE,
  };

  static auto const NUM_SHAPES = SHAPE_NOISE;

  void SetFrequency(float fFreq);
  void Sync();

  void SetWaveform(SHAPE eShape);
  void SetPulseWidth(float fPulseWidth);

  float Work();

private:
  WaveForm m_Pulse = WaveForm(WaveForm::SQUARE);
  WaveForm m_Sawtooth = WaveForm(WaveForm::SAWTOOTH);
  SineWaveForm m_Sine;
  TriangleWaveForm m_Triangle;
  NoiseWaveForm m_Noise;

  IWaveForm* m_pCurrWaveForm = &m_Sawtooth;
};

inline char const* ToString(Osc::SHAPE eShape)
{
  switch(eShape)
  {
  case Osc::SHAPE_SINE: return "sine";
  case Osc::SHAPE_TRIANGLE: return "triangle";
  case Osc::SHAPE_PULSE: return "pulse";
  case Osc::SHAPE_SAW: return "saw";
  case Osc::SHAPE_NOISE: return "noise";
  }

  return "?";
}

