/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#include "filter.h"
#include "moogfilter.h"
#include "envelope.h"
#include "osc.h"
#include "audio_buffer.h"

class Voice
{
public:
  Voice();
  void SetSampleRate(float fSampleRate);

  bool IsAlive() const;
  void On(float fFrequency);
  void Off();
  void NewChunk(float const lfo[6]);
  void Mix(AudioBuffer* buffer);

  void SetADSR(int iEnvIdx, float A, float D, float S, float R);
  void SetEnvToPitch(int iEnvIdx, float fAmount);
  void SetEnvToFilter(int iEnvIdx, float fEnvToFilter);
  void SetEnvToVol(int iEnvIdx, float fEnvToVol);
  void SetBaseFilter(int iFilterIdx, float fCutoff, float fResonance);
  void SetFilterBalance(float fBalance);
  void SetFilterPan(int iFilterIdx, float fPan);

  void SetOscVol(float fVal) { m_fOscVol = fVal; }

  void SetOscBalance(float fVal) { m_fOscBalance = fVal; }

  void SetOscRingMod(float fVal) { m_fOscRingMod = fVal; }

  void SetShape(int iOscIdx, Osc::SHAPE eShape);
  void SetDetune(int iOscIdx, float fDetune);
  void SetPulseWidth(int iOscIdx, float fPulseWidth);
  void SetPortamento(float fPortaTimeInChunks);
  void BendTo(float fBend);

private:
  bool m_bAlive;

  float getCurrPitch() const
  {
    return m_fPitch + m_fPortamentoPitch;
  }

  Envelope m_Envelope[2];

  float m_fSampleRate;
  Osc m_Osc[2];
  float m_fOscDetune[2];
  float m_fOscVol;
  float m_fOscBalance;
  float m_fOscRingMod;
  MoogFilter m_Filter[2];
  float m_fEnvToFilter[2];
  float m_fEnvToPitch[2];
  float m_fEnvToVol[2];
  float m_fFilterBalance;
  float m_fFilterPan[2];
  float m_fCutoff[2];
  float m_fResonance[2];
  float m_fPitch;
  float m_fBend;
  float m_fPortaTimeInChunks;
  float m_fPortamentoPitch; // delta pitch due to portamento
};

