/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#include "voice_dispatcher.h"
#include "voice.h"
#include <algorithm>

struct Table
{
  float freqs[128] {};
};

constexpr Table ComputeMidiFrequencies()
{
  Table r;

  for(int i = 0; i < 128; ++i)
    r.freqs[i] = float(pow(2.0, float(i - 69) / 12.0) * 440.0);

  return r;
}

static const auto g_Midi = ComputeMidiFrequencies();

int VoiceDispatcher::AllocateVoice()
{
  if(m_isMono)
    return 0;

  for(size_t i = 0; i < m_NumVoices; ++i)
    if(!m_Voices[i].IsAlive())
      return i;

  return -1; // no voice available
}

void VoiceDispatcher::NoteOn(int iNote, int /*vel*/)
{
  int iPrevNote = iNote;

  // portamento
  if(!m_PlayedNotes.empty())
    iPrevNote = m_PlayedNotes.back();

  m_PlayedNotes.push_back(iNote);

  int iVoice = AllocateVoice();

  if(iVoice < 0)
    return; // polyphony is maxed out

  m_NoteToVoice[iNote] = iVoice;
  m_Voices[iVoice].On(g_Midi.freqs[iPrevNote]);
  m_Voices[iVoice].On(g_Midi.freqs[iNote]);
}

void VoiceDispatcher::NoteOff(int iNote, int /*vel*/)
{
  bool bWasLastPlayed = false;

  if(!m_PlayedNotes.empty() && iNote == m_PlayedNotes.back())
    bWasLastPlayed = true;

  auto newEnd = std::remove(m_PlayedNotes.begin(), m_PlayedNotes.end(), iNote);
  m_PlayedNotes.erase(newEnd, m_PlayedNotes.end());

  if(m_isMono)
  {
    if(bWasLastPlayed)
    {
      // check if there's still one pressed key. If yes, switch back to it.
      if(!m_PlayedNotes.empty())
      {
        m_Voices[0].On(g_Midi.freqs[iNote]);
        m_Voices[0].On(g_Midi.freqs[m_PlayedNotes.back()]);
      }
      else
      {
        m_Voices[0].Off();
      }
    }

    for(size_t i = 1; i < m_NumVoices; ++i)
      m_Voices[i].Off();
  }
  else if(m_NoteToVoice.find(iNote) != m_NoteToVoice.end())
  {
    auto voiceIndex = m_NoteToVoice[iNote];
    m_Voices[voiceIndex].Off();
  }

  m_NoteToVoice.erase(iNote);
}

