/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#if EVEREST_ENABLE_GUI

#if _WIN32
#include <windows.h>
#endif

#include "lib_juce/juce.h"

#include "vst_adapter.h"
#include "editor_wrapper.h"

#if _WIN32

static HWND findMDIParentOf(HWND w)
{
  const int frameThickness = GetSystemMetrics(SM_CYFIXEDFRAME);

  while(w != 0)
  {
    HWND parent = GetParent(w);

    if(parent == 0)
      break;

    TCHAR windowType[32];
    zeromem(windowType, sizeof(windowType));
    GetClassName(parent, windowType, 31);

    if(String(windowType).equalsIgnoreCase(T("MDIClient")))
    {
      w = parent;
      break;
    }

    RECT windowPos;
    GetWindowRect(w, &windowPos);

    RECT parentPos;
    GetWindowRect(parent, &parentPos);

    const int dw = (parentPos.right - parentPos.left) - (windowPos.right - windowPos.left);
    const int dh = (parentPos.bottom - parentPos.top) - (windowPos.bottom - windowPos.top);

    if(dw > 100 || dh > 100)
      break;

    w = parent;

    if(dw == 2 * frameThickness)
      break;
  }

  return w;
}

#endif

namespace EVEREST
{
#if _WIN32

void EditorWrapper::mouseDown(const MouseEvent &)
{
  broughtToFront();
}

void EditorWrapper::broughtToFront()
{
  // for hosts like nuendo, need to also pop the MDI container to the
  // front when our comp is clicked on.
  HWND parent = findMDIParentOf((HWND)getWindowHandle());

  if(parent != 0)
    SetWindowPos(parent, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
}

#endif

// ==============================================================================
void EditorWrapper::childBoundsChanged(Component* child)
{
  child->setTopLeftPosition(0, 0);

  const int cw = child->getWidth();
  const int ch = child->getHeight();

// wrapper->resizeHostWindow (cw, ch);

  setSize(cw, ch);

#if JUCE_MAC
// wrapper->resizeHostWindow (cw, ch);  // (doing this a second time seems to be necessary in tracktion)
#endif
}

void EditorWrapper::handleAsyncUpdate()
{
  // wrapper->tryMasterIdle();
}
}

#endif

