/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#if EVEREST_ENABLE_GUI

#include "lib_juce/juce.h"

namespace EVEREST
{
// ==============================================================================
// A component to hold the AudioProcessorEditor, and cope with some housekeeping
// chores when it changes or repaints.
class EditorWrapper : public Component, public AsyncUpdater
{
public:
  EditorWrapper(Component* const editor)
  {
    setOpaque(true);
    editor->setOpaque(true);

    setBounds(editor->getBounds());
    editor->setTopLeftPosition(0, 0);
    addAndMakeVisible(editor);

#if _WIN32
    addMouseListener(this, true);
#endif
  }

  ~EditorWrapper()
  {
    deleteAllChildren();
  }

  void paint(Graphics & /*g*/)
  {
  }

  void paintOverChildren(Graphics & /*g*/)
  {
    // this causes an async call to masterIdle() to help
    // creaky old DAWs like Nuendo repaint themselves while we're
    // repainting. Otherwise they just seem to give up and sit there
    // waiting.
    triggerAsyncUpdate();
  }

#if EVEREST_MAC
  bool keyPressed(const KeyPress& kp)
  {
    // If we have an unused keypress, move the key-focus to a host window
    // and re-inject the event..
    forwardCurrentKeyEventToHost(this);
    return true;
  }

#endif

  void resized()
  {
    Component* const c = getChildComponent(0);

    if(c != 0)
      c->setBounds(0, 0, getWidth(), getHeight());
  }

  void childBoundsChanged(Component* child);
  void handleAsyncUpdate();

#if _WIN32
  void mouseDown(const MouseEvent &);
  void broughtToFront();
#endif
};
}

#endif

