/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#include "lib_juce/juce.h"
#include "lib_utils/math_tools.h"
#include "gui_knob.h"

static const float KNOB_RADIUS = 7.0f;
static const float PI = 3.141592654f;

namespace EVEREST
{
KnobComponent::KnobComponent(KnobListener* pListener) :
  m_pListener(pListener),
  m_fPosition(0.5),
  m_bDragging(false)
{
  setBounds(0, 0, (int)KNOB_RADIUS * 2, (int)KNOB_RADIUS * 2);
}

void KnobComponent::paint(Graphics& g)
{
  float const fAngularRange = 2.0f * PI * 0.75f;
  float const zero_position_angle = 0.5f * PI + (2.0f * PI - fAngularRange) * 0.5f;

  float fParam = m_fPosition;
  float knob_angle = fParam * fAngularRange;
  float angle = zero_position_angle + knob_angle;

  float const x1 = 0;
  float const y1 = 0;

  float const x = KNOB_RADIUS;
  float const y = KNOB_RADIUS;

  g.setColour(juce::Colour(128, 128, 128));
  g.fillEllipse(x1, y1, KNOB_RADIUS * 2, KNOB_RADIUS * 2);
  g.setColour(juce::Colour(255, 255, 255));

  float x2 = x + (float)cos(angle) * KNOB_RADIUS + 1;
  float y2 = y + (float)sin(angle) * KNOB_RADIUS + 1;
  g.drawLine(x, y, x2, y2, 2);
}

void KnobComponent::mouseDown(const MouseEvent &)
{
  m_fDragStartPosition = m_fPosition;
  m_bDragging = true;
}

void KnobComponent::mouseUp(const MouseEvent &)
{
  m_bDragging = false;
}

void KnobComponent::mouseDrag(const MouseEvent& e)
{
  float fNewPosition = (-e.getDistanceFromDragStartY()) * 0.005f + m_fDragStartPosition;
  setPosition(fNewPosition);

  if(m_pListener)
    m_pListener->OnValueChanged(this);
}

bool KnobComponent::IsDragging() const
{
  return m_bDragging;
}

void KnobComponent::setPosition(float fPosition)
{
  fPosition = Clamp(fPosition, 0.0f, 1.0f);

  if(fPosition != m_fPosition)
  {
    m_fPosition = fPosition;
    repaint();
  }
}

float KnobComponent::GetPosition() const
{
  return m_fPosition;
}
}

