/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#include "lib_juce/juce.h"
#include "gui_knob2.h"
#include "knob_png.h"

EverestParamKnob::EverestParamKnob(ParamAccess access) :
  juce::Slider(T("Slider")),
  m_setParam(access.setParam),
  m_getParam(access.getParam)
{
  setRange(0, 1, 0);
  setSliderStyle(Slider::RotaryVerticalDrag);
  setTextBoxStyle(Slider::NoTextBox, true, 20, 20);
  setColour(Slider::backgroundColourId, Colour(0xffffffff));
  setColour(Slider::rotarySliderFillColourId, Colour(0xff0064c5));
  setColour(Slider::rotarySliderOutlineColourId, Colour(0xff888888));
  addListener(this);
  startTimer(75);
}

void EverestParamKnob::paint(Graphics& g)
{
  int const NUM_FRAMES = 64;
  int index = NUM_FRAMES * getValue();
  index = std::min(NUM_FRAMES - 1, index);
  index = std::max(0, index);

  juce::Image m_imageKnob(juce::ImageCache::getFromMemory(knob_png, sizeof knob_png));
  int srcY = index * 32;
  g.drawImage(m_imageKnob, 4, 5, 32, 32, 0, srcY, 32, 32);
}

void EverestParamKnob::timerCallback()
{
  // if(!isDragging())
  {
    float fPos = m_getParam();
    setValue(fPos, false, false);
  }
}

void EverestParamKnob::sliderValueChanged(juce::Slider*)
{
  m_setParam(getValue());
}

