// Copyright (C) 2018 - Sebastien Alaiwan
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.

// Main include file of eVereST.
// Defines the interfaces that your plugin must/might implement.

#pragma once

#include <stdint.h>
#include <string>
#include <vector>

#if EVEREST_ENABLE_GUI
namespace juce
{
class Component;
}
#endif

namespace EVEREST
{
struct IHost
{
  virtual ~IHost() = default;

  virtual void OnParamChanged(int iIndex, float fValue) = 0;
};

struct ISerializable
{
  virtual std::vector<uint8_t> saveState() const = 0;
  virtual void loadState(std::vector<uint8_t> const& data) = 0;
};

struct IMidiListener
{
  virtual void noteOn(int iChannel, int iNote, int iVelocity) = 0;
  virtual void noteOff(int iChannel, int iNote, int iVelocity) = 0;
  virtual void pitchBend(int iChannel, float fBend) = 0;
  virtual void controlChange(int iChannel, int iControl, int iValue) = 0;
};

struct IPlugin
{
  virtual ~IPlugin() = default;

  virtual void SetSampleRate(float fSampleRate) = 0;
  virtual void ProcessAudio(float** pInput, float** pOutput, long iNumSamples) = 0;

  // midi events
  virtual void OnTempoChange(float fTempo) = 0;

  // meta
  virtual void GetId(char* pId) const = 0;

  // params
  virtual int GetNumParams() const { return 0; }

  virtual std::string GetParamName(int /*iParam*/) const { return "NoName"; }

  virtual float GetParam(int /*iIndex*/) const { return .0f; }

  virtual std::string GetParamText(int /*iIndex*/) const { return "NoText"; }

  virtual void SetParam(int /*iIndex*/, float /*fValue*/) {}
};

struct PluginDescription
{
  bool isSynth;
  struct
  {
    int width, height;
  } size;
  const char* author;
  const char* name;
  IPlugin* (* create)(IHost*);

#if EVEREST_ENABLE_GUI
  juce::Component* (*createEditor)(IPlugin*, IHost*);
#endif
};

// user-defined
extern const PluginDescription g_PlugDesc;
}

