// Reversed engineered VST version 2 compatibility header.
// This file is based on FL Studio protocol analysis. Please let me know
// if your host does not conform to it.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#include <string>
#include <sstream>

static const int VST_PLUGFLAG_HASEDITOR = 1 << 0;
static const int VST_PLUGFLAG_CANDOREPLACING = 1 << 4;
static const int VST_PLUGFLAG_SAVE_STATE = 1 << 5;
static const int VST_PLUGFLAG_SYNTH = 1 << 8;

enum VST_CMD
{
  VST_CMD_OPEN = 0,
  VST_CMD_CLOSE = 1,
  VST_CMD_GETPATCHNAME = 5,
  VST_CMD_GETPARAMUNITS = 6,
  VST_CMD_GETPARAMSTRINGVALUE = 7,
  VST_CMD_GETPARAMNAME = 8,
  VST_CMD_SETSAMPLERATE = 10,
  VST_CMD_GETRECT = 13,
  VST_CMD_SHOW = 14,
  VST_CMD_HIDE = 15,
  VST_CMD_SAVE = 23,
  VST_CMD_LOAD = 24,
  VST_CMD_PROCESSEVENTS = 25,
  VST_CMD_ISAUTOMATABLE = 26,
  VST_CMD_GETPLUGNAME = 45,
  VST_CMD_GETAUTHORNAME = 47,
  VST_CMD_GETPRODUCTNAME = 48,
  VST_CMD_GETCAPABILITY = 51,
  VST_CMD_GETNUMMIDIINPUTS = 78,
  VST_CMD_GETNUMMIDIOUTPUTS = 79,
  VST_CMD_SETTEMPO = 65536,
};

inline std::string ToString(VST_CMD cmd)
{
  switch(cmd)
  {
  case VST_CMD_OPEN: return "VST_CMD_OPEN";
  case VST_CMD_CLOSE: return "VST_CMD_CLOSE";
  case VST_CMD_GETPATCHNAME: return "VST_CMD_GETPATCHNAME";
  case VST_CMD_SETSAMPLERATE: return "VST_CMD_SETSAMPLERATE";
  case VST_CMD_GETRECT: return "VST_CMD_GETRECT";
  case VST_CMD_SHOW: return "VST_CMD_SHOW";
  case VST_CMD_HIDE: return "VST_CMD_HIDE";
  case VST_CMD_PROCESSEVENTS: return "VST_CMD_PROCESSEVENTS";
  case VST_CMD_ISAUTOMATABLE: return "VST_CMD_ISAUTOMATABLE";
  case VST_CMD_GETPLUGNAME: return "VST_CMD_GETPLUGNAME";
  case VST_CMD_SAVE: return "VST_CMD_SAVE";
  case VST_CMD_GETPARAMUNITS: return "VST_CMD_GETPARAMUNITS";
  case VST_CMD_GETPARAMNAME: return "VST_CMD_GETPARAMNAME";
  case VST_CMD_GETPARAMSTRINGVALUE: return "VST_CMD_GETPARAMSTRINGVALUE";
  case VST_CMD_LOAD: return "VST_CMD_LOAD";
  case VST_CMD_GETPRODUCTNAME: return "VST_CMD_GETPRODUCTNAME";
  case VST_CMD_GETCAPABILITY: return "VST_CMD_GETCAPABILITY";
  case VST_CMD_GETNUMMIDIINPUTS: return "VST_CMD_GETNUMMIDIINPUTS";
  case VST_CMD_GETNUMMIDIOUTPUTS: return "VST_CMD_GETNUMMIDIOUTPUTS";
  case VST_CMD_SETTEMPO: return "VST_CMD_SETTEMPO";
  default:
    {
      std::stringstream s;
      s << "VST_CMD_" << cmd;
      return s.str();
    }
  }
}

enum VST_EVT
{
  VST_EVT_MIDI = 1,
};

struct VstMidiEvent
{
  /* 0x00 */
  int m_iType;
  /* 0x04 */ int m_iSize;
  /* 0x08 */ int deltaFrames;
  /* 0x0C */ int flags;
  /* 0x10 */ int noteLength;
  /* 0x14 */ int noteOffset;
  /* 0x18 */ char midiData[4];
  /* 0x1C */ char detune;
  /* 0x1D */ char noteOffVelocity;
  /* 0x1E */ char unknown1;
  /* 0x1F */ char unknown2;
};

struct VstEvent
{
  union
  {
    int m_iType;
    VstMidiEvent midi;
  };
};

struct VstEvents
{
  /* 0x00 */
  int numEvents;
  /* 0x04 */ void* unknown;
  /* 0x08 */ VstEvent* events[1];
};

struct VstTime
{
  double unknown1[4];
  double tempo;
  double unknown2[3];
  int unknown3[6];
};

struct VstRect
{
  /* 0x00 */
  short x;
  /* 0x02 */ short y;
  /* 0x04 */ short cy;
  /* 0x06 */ short cx;
};

#ifdef _MSC_VER
#define VSTAPI __cdecl
#else
#define VSTAPI
#endif

// Do not change the order of the members!!
struct IVst2Plugin
{
  /* 0x00 */
  char magic[4];
  /* 0x04 */ int(VSTAPI * dispatcher)(IVst2Plugin * pThis, int, int, intptr_t, void*, float);
  /* 0x08 */ void(VSTAPI * process)(IVst2Plugin * pThis, float**, float**, int);
  /* 0x0C */ void(VSTAPI * setParameter)(IVst2Plugin * pThis, int, float);
  /* 0x10 */ float(VSTAPI * getParameter)(IVst2Plugin * pThis, int);
  /* 0x14 */ int numPrograms;
  /* 0x18 */ int numParams;
  /* 0x1C */ int numInputs;
  /* 0x20 */ int numOutputs;
  /* 0x24 */ int flags;
  /* 0x28 */ void* ptr1;
  /* 0x2C */ void* ptr2;
  /* 0x30 */ char empty3[4 + 4 + 4];
  /* 0x3C */ float unkown_float;
  /* 0x40 */ void* ptr3;
  /* 0x44 */ void* userdata;
  /* 0x48 */ char plugin_id[4];
  /* 0x4C */ char unknown1[4];
  /* 0x50 */ void(VSTAPI * processReplacing)(IVst2Plugin*, float**, float**, int);
};

typedef intptr_t (VSTAPI * HostControlFunction)(IVst2Plugin* pThis, long int, long int,
                                                long int, void*, float);

