// Copyright (C) 2018 - Sebastien Alaiwan
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as
// published by the Free Software Foundation, either version 3 of the
// License, or (at your option) any later version.

// VST-adapter for Everest plugins

#pragma once

#include "lib_everest.h"
#include "vst2_interface.h"

namespace EVEREST
{
IVst2Plugin* CreateVst2Plugin(HostControlFunction cb);
}

namespace EVEREST
{
class VstAdapter : public IHost, public IVst2Plugin
{
public:
  VstAdapter(HostControlFunction hostDo);
  void Init(IPlugin* pPlugin);
  virtual ~VstAdapter();

private:
#if EVEREST_ENABLE_GUI
  juce::Component * m_pEditorComp;
#endif
  VstRect m_Rect;
  IPlugin* m_Plugin;
  HostControlFunction m_HostDo;
  std::vector<uint8_t> m_StateData;

  // IHost implementation
  void OnParamChanged(int iIndex, float fValue);

#define WRAPPER_PROLOGUE auto pWrapper = static_cast<VstAdapter*>(pVstInterface)

  /***********************************************************************/
  /*                                                                     */
  /***********************************************************************/
  static int static_dispatcher(IVst2Plugin* pVstInterface, int a, int b, intptr_t c, void* d, float e)
  {
    WRAPPER_PROLOGUE;
    return pWrapper->Dispatcher(VST_CMD(a), b, c, d, e);
  }

  static void static_setParameter(IVst2Plugin* pVstInterface, int iParamId, float fValue)
  {
    WRAPPER_PROLOGUE;
    pWrapper->SetParameter(iParamId, fValue);
  }

  static float static_getParameter(IVst2Plugin* pVstInterface, int iParamId)
  {
    WRAPPER_PROLOGUE;
    return pWrapper->GetParameter(iParamId);
  }

  static void static_process(IVst2Plugin* pVstInterface, float** pInput, float** pOutput, int iNumSamples)
  {
    WRAPPER_PROLOGUE;
    pWrapper->Process(pInput, pOutput, iNumSamples);
  }

  static void static_processReplacing(IVst2Plugin* pVstInterface, float** pInput, float** pOutput, int iNumSamples)
  {
    WRAPPER_PROLOGUE;
    pWrapper->Process(pInput, pOutput, iNumSamples);
  }

  /***********************************************************************/
  /*                                                                     */
  /***********************************************************************/

#if EVEREST_ENABLE_GUI
  void CreateEditorComponent();
#endif

  /***********************************************************************/

  int Dispatcher(VST_CMD eRequest, int iParam1, intptr_t iParam2, void* pData, float fParam);

  void SetParameter(int iParamIdx, float fValue)
  {
    if(iParamIdx < 0 || iParamIdx >= m_Plugin->GetNumParams())
      return;

    m_Plugin->SetParam(iParamIdx, fValue);
  }

  float GetParameter(int iParamIdx)
  {
    if(iParamIdx < 0 || iParamIdx >= m_Plugin->GetNumParams())
      return 0;

    return m_Plugin->GetParam(iParamIdx);
  }

  void ProcessMidiEvent(VstMidiEvent* e);
  void Process(float** pInput, float** pOutput, int iNumSamples);

  double GetTempo();
  void LoadPluginState();
  void SavePluginState();

  std::vector<VstMidiEvent> m_EventBuffer;

  double m_fTempo;
};
}

