#pragma once

 #if JUCE_WINDOWS

/*** Start of inlined file: juce_win32_NativeIncludes.h ***/
#ifndef __JUCE_WIN32_NATIVEINCLUDES_JUCEHEADER__
#define __JUCE_WIN32_NATIVEINCLUDES_JUCEHEADER__

#ifndef STRICT
 #define STRICT 1
#endif

#undef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN 1

#define _WIN32_WINNT 0x0502
#define _WIN32_IE 0x0501
#define _UNICODE 1
#define UNICODE 1

#include <windows.h>
#include <windowsx.h>
#include <commdlg.h>
#include <shellapi.h>
#include <mmsystem.h>
#include <vfw.h>
#include <tchar.h>
#include <stddef.h>
#include <ctime>
#include <wininet.h>
#include <nb30.h>
#include <iphlpapi.h>
#include <mapi.h>
#include <float.h>
#include <process.h>
#include <exdispid.h>
#include <shlobj.h>
#include <shlwapi.h>
#include <imm.h>

#if ! JUCE_MINGW
 #include <crtdbg.h>
 #include <comutil.h>
#endif

#undef PACKED

#if JUCE_MSVC
 #pragma warning (pop)
#endif

#ifndef WM_APPCOMMAND
 #define WM_APPCOMMAND 0x0319
#endif


/*** Start of inlined file: juce_win32_ComSmartPtr.h ***/
#ifndef __JUCE_WIN32_COMSMARTPTR_JUCEHEADER__
#define __JUCE_WIN32_COMSMARTPTR_JUCEHEADER__

#define JUCE_COMRESULT  HRESULT __stdcall

/** Handy base class for writing COM objects, providing ref-counting and a basic QueryInterface method.
*/
template <class ComClass>
class ComBaseClassHelper   : public ComClass
{
public:
	ComBaseClassHelper()  : refCount (1) {}
	virtual ~ComBaseClassHelper() {}

	JUCE_COMRESULT QueryInterface (REFIID refId, void** result)
	{
	   #ifndef __MINGW32__
		if (refId == __uuidof (ComClass))   { AddRef(); *result = dynamic_cast <ComClass*> (this); return S_OK; }
	   #endif

		if (refId == IID_IUnknown)	  { AddRef(); *result = dynamic_cast <IUnknown*> (this); return S_OK; }

		*result = 0;
		return E_NOINTERFACE;
	}

	ULONG __stdcall AddRef()	{ return ++refCount; }
	ULONG __stdcall Release()   { const int r = --refCount; if (r == 0) delete this; return r; }

protected:
	int refCount;
};

#endif   // __JUCE_WIN32_COMSMARTPTR_JUCEHEADER__

/*** End of inlined file: juce_win32_ComSmartPtr.h ***/

/* Used with DynamicLibrary to simplify importing functions

   functionName: function to import
   localFunctionName: name you want to use to actually call it (must be different)
   returnType: the return type
   object: the DynamicLibrary to use
   params: list of params (bracketed)
*/
#define JUCE_DLL_FUNCTION(functionName, localFunctionName, returnType, object, params) \
	typedef returnType (WINAPI *type##localFunctionName) params; \
	type##localFunctionName localFunctionName  \
		= (type##localFunctionName)object.getFunction (#functionName);

#endif   // __JUCE_WIN32_NATIVEINCLUDES_JUCEHEADER__

/*** End of inlined file: juce_win32_NativeIncludes.h ***/


 #elif JUCE_LINUX

/*** Start of inlined file: juce_linux_NativeIncludes.h ***/
#ifndef __JUCE_LINUX_NATIVEINCLUDES_JUCEHEADER__
#define __JUCE_LINUX_NATIVEINCLUDES_JUCEHEADER__

/*
	This file wraps together all the linux-specific headers, so
	that we can include them all just once, and compile all our
	platform-specific stuff in one big lump, keeping it out of the
	way of the rest of the codebase.
*/

#include <sched.h>
#include <pthread.h>
#include <sys/time.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/dir.h>
#include <sys/ptrace.h>
#include <sys/vfs.h>
#include <sys/wait.h>
#include <sys/mman.h>
#include <unistd.h>
#include <fnmatch.h>
#include <utime.h>
#include <pwd.h>
#include <fcntl.h>
#include <dlfcn.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <net/if.h>
#include <sys/sysinfo.h>
#include <sys/file.h>
#include <sys/prctl.h>
#include <signal.h>

/* Got a build error here? You'll need to install the freetype library...
   The name of the package to install is "libfreetype6-dev".
*/
#include <ft2build.h>
#include FT_FREETYPE_H

#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xresource.h>
#include <X11/Xutil.h>
#include <X11/Xmd.h>
#include <X11/keysym.h>
#include <X11/cursorfont.h>
#include <X11/XKBlib.h>

#if JUCE_USE_XINERAMA
 /* If you're trying to use Xinerama, you'll need to install the "libxinerama-dev" package..  */
 #include <X11/extensions/Xinerama.h>
#endif

#if JUCE_USE_XSHM
 #include <X11/extensions/XShm.h>
 #include <sys/shm.h>
 #include <sys/ipc.h>
#endif

#if JUCE_USE_XRENDER
 // If you're missing these headers, try installing the libxrender-dev and libxcomposite-dev
 #include <X11/extensions/Xrender.h>
 #include <X11/extensions/Xcomposite.h>
#endif

#if JUCE_USE_XCURSOR
 // If you're missing this header, try installing the libxcursor-dev package
 #include <X11/Xcursor/Xcursor.h>
#endif

#undef KeyPress

#if JUCE_JACK
 /* Got an include error here? If so, you've either not got jack-audio-connection-kit
	installed, or you've not got your paths set up correctly to find its header files.

	The package you need to install to get JACK support is "libjack-dev".

	If you don't have the jack-audio-connection-kit library and don't want to build
	Juce with low latency audio support, just disable the JUCE_JACK flag in juce_Config.h
 */
 #include <jack/jack.h>
 //#include <jack/transport.h>
#endif

#undef SIZEOF

#endif   // __JUCE_LINUX_NATIVEINCLUDES_JUCEHEADER__

/*** End of inlined file: juce_linux_NativeIncludes.h ***/


 #else
  #error "Unknown platform!"
 #endif
