#pragma once

#include <windows.h>
#include <string>
#include <vector>

#include "ksobject.h"

class CKsPin;

typedef std::basic_string<TCHAR> tstring;

class CKsFilter :
  public CKsObject
{
  friend class CKsPin;

public:
  CKsFilter(tstring const& name, std::string const& sFriendlyName);
  virtual ~CKsFilter();

  virtual void Instantiate();
  virtual void EnumeratePins();

  template<typename T>
  HRESULT GetPinProperty(ULONG nPinId, REFGUID guidPropertySet, ULONG nProperty, T& value)
  {
    ULONG ulReturned = 0;
    KSP_PIN ksPProp;

    ksPProp.Property.Set = guidPropertySet;
    ksPProp.Property.Id = nProperty;
    ksPProp.Property.Flags = KSPROPERTY_TYPE_GET;
    ksPProp.PinId = nPinId;
    ksPProp.Reserved = 0;

    HRESULT hr = DeviceIoControl(
      m_handle,
      IOCTL_KS_PROPERTY,
      &ksPProp,
      sizeof(KSP_PIN),
      &value,
      sizeof(value),
      &ulReturned,
      NULL);

    if(FAILED(hr))
    {
      return hr;
    }

    return S_OK;
  }

  HRESULT GetPinPropertyMulti(IN ULONG nPinID, IN REFGUID guidPropertySet, IN ULONG nProperty, OUT PKSMULTIPLE_ITEM* ppKsMultipleItem OPTIONAL);

  tstring const& GetName() const
  {
    return m_sName;
  }

  std::string const& GetFriendlyName() const
  {
    return m_sFriendlyName;
  }

protected:
  void ClassifyPins();

  // this list owns the pins.
  std::vector<CKsPin*> m_listPins;

  std::vector<CKsPin*> m_listRenderSinkPins;
  std::vector<CKsPin*> m_listRenderSourcePins;
  std::vector<CKsPin*> m_listCaptureSinkPins;
  std::vector<CKsPin*> m_listCaptureSourcePins;
  std::vector<CKsPin*> m_listNoCommPins;

private:
  std::string const m_sFriendlyName; // friendly name eg "Virus TI Synth"
  tstring const m_sName; // Filter path, eg "\\?\usb#vid_133e&pid_0815...\vtimidi02"
};

