/**
 * @file ksobject.cpp
 * @author Sebastien Alaiwan
 * @date 2011-11-20
 */

/*
 * Copyright (C) 2011 - Sebastien Alaiwan
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <cassert>
#include <windows.h>
#include "ddk/ks.h"
#include "ddk/ksmedia.h"
#include "ksobject.h"
#include "util.h"

CKsObject::CKsObject(HANDLE handle) :
  m_handle(handle)
{
}

HANDLE CKsObject::GetHandle() const
{
  return m_handle;
}

HRESULT CKsObject::SetProperty(REFGUID guidPropertySet, ULONG nProperty, void* pvValue, ULONG cbValue)
{
  KSPROPERTY ksProperty;
  Zero(ksProperty);
  ksProperty.Set = guidPropertySet;
  ksProperty.Id = nProperty;
  ksProperty.Flags = KSPROPERTY_TYPE_SET;

  HRESULT hr = DeviceIoControlKsProperty(ksProperty, pvValue, cbValue);

  if(FAILED(hr))
  {
    return hr;
  }

  return S_OK;
}

HRESULT CKsObject::GetProperty(REFGUID guidPropertySet, ULONG nProperty, void* pvValue, ULONG cbValue)
{
  KSPROPERTY ksProperty;
  Zero(ksProperty);
  ksProperty.Set = guidPropertySet;
  ksProperty.Id = nProperty;
  ksProperty.Flags = KSPROPERTY_TYPE_GET;

  HRESULT hr = DeviceIoControlKsProperty(ksProperty, pvValue, cbValue);

  if(FAILED(hr))
  {
    return hr;
  }

  return S_OK;
}

HRESULT CKsObject::GetPropertyMulti(REFGUID guidPropertySet, ULONG nProperty, KSMULTIPLE_ITEM** ppKsMultipleItem)
{
  KSPROPERTY ksProperty;
  Zero(ksProperty);
  ksProperty.Set = guidPropertySet;
  ksProperty.Id = nProperty;
  ksProperty.Flags = KSPROPERTY_TYPE_GET;

  if(ppKsMultipleItem == NULL)
  {
    return E_POINTER;
  }

  *ppKsMultipleItem = NULL;

  ULONG iNumItems = 0;
  HRESULT hr;
  hr = DeviceIoControl(
    m_handle,
    IOCTL_KS_PROPERTY,
    &ksProperty,
    sizeof(KSPROPERTY),
    NULL,
    0,
    &iNumItems,
    NULL);

  if(FAILED(hr))
  {
    return hr;
  }

  if(iNumItems)
  {
    *ppKsMultipleItem = (PKSMULTIPLE_ITEM) new BYTE[iNumItems];
    ULONG ulReturned = 0;
    hr = DeviceIoControl(
      m_handle,
      IOCTL_KS_PROPERTY,
      &ksProperty,
      sizeof(ksProperty),
      reinterpret_cast<VOID*>(*ppKsMultipleItem),
      iNumItems,
      &ulReturned,
      NULL);
  }

  return hr;
}

