#pragma once

#include <windows.h>
#include "ddk/ks.h"
#include "ddk/ksmedia.h"

#include "ksobject.h"
#include "ksfilter.h"

struct PIN_DESCRIPTOR
{
  ULONG cInterfaces;
  PKSIDENTIFIER pInterfaces;
  ULONG cMediums;
  PKSIDENTIFIER pMediums;
  ULONG cDataRanges;
  PKSDATARANGE pDataRanges;
  ULONG cConstrDataRanges;
  PKSDATARANGE pConstrDataRanges;
  KSPIN_DATAFLOW DataFlow;
  KSPIN_COMMUNICATION Communication;
  KSPIN_CINSTANCES CInstances;
  KSPIN_CINSTANCES CInstancesGlobal;
  KSPIN_CINSTANCES CInstancesNecessary;
  PKSPIN_PHYSICALCONNECTION PhysicalConnection;
  GUID Category;
  PWCHAR Name;

  // these are redundant.  Free them when you are done with pInterfaces, pMediums, pDataRanges
  PKSMULTIPLE_ITEM pmiDataRanges;
  PKSMULTIPLE_ITEM pmiMediums;
  PKSMULTIPLE_ITEM pmiInterfaces;
};

typedef PIN_DESCRIPTOR* PPIN_DESCRIPTOR;

// This is the base class for classes that proxy Ks filters from user mode.
// The pin class is still virtual and must be overidden to be useful.
// specifically, the mpksPinCreate need to be propperly
// initialized in the derived class.

class CKsPin :
  public CKsObject
{
public:
  CKsPin(CKsFilter* pFilter, ULONG nId);
  virtual ~CKsPin();

  virtual HRESULT Instantiate(bool fLooped = false);

  void ClosePin();

  HRESULT SetFormat(KSDATAFORMAT* pFormat);

  virtual void SetState(KSSTATE ksState);
  void GetState(KSSTATE& pksState);

  void WriteData(KSSTREAM_HEADER* pKSSTREAM_HEADER, OVERLAPPED* pOVERLAPPED);
  void ReadData(KSSTREAM_HEADER* pKSSTREAM_HEADER, OVERLAPPED* pOVERLAPPED);

  ULONG GetId() const;

  HRESULT SetPinConnect(KSPIN_CONNECT const* pksPinConnect, ULONG cbKsPinConnect);

  KSPIN_DATAFLOW GetDataFlow() const;
  KSPIN_COMMUNICATION GetCommunication() const;

protected:
  DWORD m_dwAlignment;
  PKSPIN_CONNECT m_pksPinCreate;    // creation parameters of pin
  DWORD m_cbPinCreateSize; // size of the creation parameters
  ULONG m_nId;
  CKsFilter* m_pFilter;
  GUID m_guidCategory;
  PIN_DESCRIPTOR m_Descriptor;      // description of Pin

private:
  HRESULT Init();

  KSSTATE m_ksState;         // state of pin
  tstring m_szFriendlyName; // friendly name of the pin
  BOOL m_fLooped;
  CKsPin* m_pLinkPin;        // optional pin to connect to
  BOOL m_fCompleteOnlyOnRunState;
};

