/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#pragma once

#include <vector>
#include <string>
#include "string_format.h"

namespace bytestream
{
class end_of_stream
{
};
}

class ibytestream
{
public:
  ibytestream(std::vector<uint8_t> const& _src) : src(_src), pos(0)
  {
  }

  uint8_t read_char()
  {
    if(pos >= src.size())
      throw bytestream::end_of_stream();

    char c = src[pos++];
    return c;
  }

  std::string read_string()
  {
    std::string r;

    while(true)
    {
      char c = read_char();

      if(c == 0)
        break;

      r += c;
    }

    return r;
  }

  unsigned int read_uint32()
  {
    unsigned int val = 0;
    val |= (read_char() << 0);
    val |= (read_char() << 8);
    val |= (read_char() << 16);
    val |= (read_char() << 24);
    return val;
  }

  std::vector<uint8_t> read_vector()
  {
    auto const iLength = read_uint32();
    std::vector<uint8_t> r;
    r.reserve(iLength);

    for(size_t i = 0; i < iLength; ++i)
      r.push_back(read_char());

    return r;
  }

private:
  std::vector<uint8_t> const& src;
  size_t pos;
};

class obytestream
{
public:
  obytestream(std::vector<uint8_t>& _dst) : dst(_dst)
  {
  }

  void write_char(uint8_t c)
  {
    dst.push_back(c);
  }

  void write_string(std::string const& s)
  {
    for(size_t i = 0; i < s.size(); ++i)
      write_char(s[i]);

    write_char(0);
  }

  void write_uint32(unsigned int i)
  {
    write_char((i >> 0) & 0xFF);
    write_char((i >> 8) & 0xFF);
    write_char((i >> 16) & 0xFF);
    write_char((i >> 24) & 0xFF);
  }

  void write_vector(std::vector<uint8_t> const& buffer)
  {
    write_buffer(&buffer[0], buffer.size());
  }

  void write_buffer(uint8_t const* pBuffer, size_t iLength)
  {
    write_uint32((unsigned int)iLength);

    for(size_t i = 0; i < iLength; ++i)
      write_char(pBuffer[i]);
  }

private:
  std::vector<uint8_t>& dst;
};

