/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#pragma once

#include <string>
#include <sstream>
#include "tools.h"

struct Fraction
{
  Fraction() :
    num(0), denom(1)
  {
  }

  Fraction(int n, int d) :
    num(n), denom(d)
  {
  }

  Fraction inv() const
  {
    return Fraction(denom, num);
  }

  bool operator == (int const& v) const
  {
    return v * denom == num;
  }

  bool operator == (Fraction const& f) const
  {
    return f.num * denom == f.denom * num;
  }

  int num, denom;
};

inline float ToFloat(Fraction const& f)
{
  return float(f.num) / float(f.denom);
}

inline std::string ToString(Fraction const& f)
{
  return format("%s/%s", f.num, f.denom);
}

