/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#pragma once

#include <cassert>

static const double MY_PI = 3.14159265358979323846;

template<typename T>
inline T Clamp(T v, T vMin, T vMax)
{
  if(v < vMin)
    return vMin;

  if(v > vMax)
    return vMax;

  return v;
}

template<typename T>
inline bool IsNan(T v)
{
  return v != v;
}

template<typename T>
inline T blend(T a, T b, T alpha)
{
  return a + alpha * (b - a);
}

template<typename TT>
struct Cycle
{
  Cycle(TT maxVal) : m_Max(maxVal)
  {
  }

  bool inc(TT val = TT(1))
  {
    assert(val <= m_Max);
    bool cycled = m_Val >= m_Max;
    m_Val += val;

    if(cycled)
      m_Val -= m_Max;

    return cycled;
  }

  TT length() const
  {
    return m_Max;
  }

private:
  TT m_Max;
  TT m_Val = TT(0);
};

