// Declaration of a software object representing a Virus Synth.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#include <string>
#include "viruspatch.h"

struct IMidiSink
{
  virtual void WriteMidi(uint8_t const* pData, int iLength) = 0;
  virtual void OnPatchReceived(int /*iPart*/, VirusPatch const & /*p*/) {}

  virtual void OnVirusParamChanged(int /*iPart*/, int /*iOffset*/, int /*iValue*/) {}
};

struct IMidiSource
{
  virtual void Flush() = 0;
  virtual bool PullSysex(std::vector<uint8_t>& sysex) = 0;
};

class CMidiVirus
{
public:
  CMidiVirus(IMidiSink* pSink = nullptr);
  virtual ~CMidiVirus() {}

  void Init(IMidiSource* pSource, IMidiSink* pSink);

  bool DownloadPatch(int iPartNumber, VirusPatch& v);
  bool UploadPatch(int iPartNumber, VirusPatch const& v);
  void RequestSingleDump(int iPartNumber);
  void ProcessMidi(uint8_t const* pData, size_t iLength);

  void GoToSequencerMode();

private:
  void SendSysex(uint8_t const* pSysex, int iLength);
  void ProcessSysex(std::vector<uint8_t> const& sysex);

  std::string m_sInputDeviceName;

  VirusPatch m_Patches[16];

  IMidiSink* m_pOutput;
  IMidiSource* m_pSource;

  enum STATE
  {
    STATE_INIT,
    STATE_SYSEX,
    STATE_CC,
  };

  STATE m_State;
  int m_StateChannel;
  std::vector<uint8_t> m_CurrentSysex;
  std::vector<uint8_t> m_CurrentData;
};

