/**
 * @file acecrusher.h
 * @brief Yet another bit crusher / rate reducer plugin.
 * @author Sebastien Alaiwan
 * @date 2010-12-01
 */

/* Copyright (C)
 * 2010 - Sebastien Alaiwan
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#pragma once

#include <string>
#include <vector>
#include "lib_everest/lib_everest.h"
#include "lib_utils/math_tools.h"
#include "dsp/crusher.h"

using namespace std;

enum
{
  PARAM_CLIP,
  PARAM_BITS,
  PARAM_RATE,
  PARAM_POSTFILT,
  PARAM_NONLIN,

  NUM_PARAMS,
};

class AceCrusherPlugin : public EVEREST::IPlugin
{
public:
  AceCrusherPlugin(EVEREST::IHost* pHost);

  static EVEREST::IPlugin* CreateInstance(EVEREST::IHost* pHost);

  // EVEREST::IPlugin implementation
  virtual void SetSampleRate(float fSampleRate);

  virtual void ProcessAudio(float** pInput, float** pOutput, long iNumSamples);

  // midi events
  virtual void OnTempoChange(float fTempo);

  virtual void GetId(char* pId) const
  {
    pId[0] = '@';
    pId[1] = '_';
    pId[2] = 'r';
    pId[3] = 'U';
  }

  // parameters
  virtual int GetNumParams() const;
  virtual string GetParamName(int iParam) const;
  virtual string GetParamText(int iIndex) const;
  virtual float GetParam(int iIndex) const;
  virtual void SetParam(int iIndex, float fValue);

private:
  float m_ParamValues[NUM_PARAMS];
  DspCrusher m_dsp;

  EVEREST::IHost* const m_pHost;
};

