#include "crusher.h"
#include <cmath> // pow
#include "lib_utils/tools.h"

DspCrusher::DspCrusher() : m_Cycle(1)
{
  ArraySet(m_buf, 0.0f);

  m_bDirty = true;
}

void DspCrusher::recompute()
{
  float const f = 0.5f - m_fParamRate;

  int const cycleLengthInSamples = (int)exp(18.0f * f);
  m_Cycle = Cycle<int>(cycleLengthInSamples);

  m_fClip = (float)(pow(10.0, (-1.5 + 1.5 * m_fParamClip)));
  fo2 = filterFreq((float)pow(10.0f, 2.30104f + 2.f * m_fParamPostFilt));
  fi2 = (1.f - fo2);
  fi2 = fi2 * fi2;
  fi2 = fi2 * fi2;
  float _g1 = (float)(pow(2.0, 2.0 + int(m_fParamBits * 12.0)));
  go = (float)(1.0 / (2.0 * _g1));

  if(m_fParamRate > 0.5)
    gi = -_g1 / (float)cycleLengthInSamples;
  else
    gi = -_g1;

  if(m_fParamNonLin > 0.5)
  {
    lin1 = (float)(pow(10.0, 0.3 * (0.5 - m_fParamNonLin)));
    lin2 = lin1;
  }
  else
  {
    lin1 = (float)pow(10.0, 0.3 * (m_fParamNonLin - 0.5));
    lin2 = 1.0;
  }

  m_bDirty = false;
}

float DspCrusher::filterFreq(float hz) const
{
  float const r = 0.999f;

  float const j = r * r - 1;
  float const k = (float)(2.f - 2.f * r * r * cos(0.647f * hz / m_fSampleRate));
  return (float)((sqrt(k * k - 4.f * j * j) - k) / (2.f * j));
}

void DspCrusher::ProcessAudio(float** pInput, float** pOutput, long iNumSamples)
{
  float const* const in1 = pInput[0];
  float const* const in2 = pInput[1];
  float* const out1 = pOutput[0];
  float* const out2 = pOutput[1];

  if(m_bDirty)
    recompute();

  float b[10];

  ArrayCopy(b, m_buf);

  for(long i = 0; i < iNumSamples; ++i)
  {
    b[0] = (in1[i] + in2[i]);

    if(m_Cycle.inc())
    {
      b[5] = (float)(go * int(b[0] * gi));

      if(b[5] > 0)
        b[5] = (float)pow(b[5], lin2);
      else
        b[5] = -(float)pow(-b[5], lin1);

      b[5] = Clamp(b[5], -m_fClip, m_fClip);

      b[0] = 0;
    }

    b[1] = fi2 * b[5] + fo2 * b[1];
    b[2] = b[1] + fo2 * b[2];
    b[3] = b[2] + fo2 * b[3];
    b[4] = b[3] + fo2 * b[4];
    b[6] = fi2 * b[4] + fo2 * b[6];
    b[7] = b[6] + fo2 * b[7];
    b[8] = b[7] + fo2 * b[8];
    b[9] = b[8] + fo2 * b[9];

    out1[i] = b[9];
    out2[i] = b[9];
  }

  if(fabs(b[1]) < 1.0e-10)
    ArraySet(m_buf, 0.0f);
  else
    ArrayCopy(m_buf, b);
}

