/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#include "lib_utils/fraction.h"
#include "lib_everest/lib_everest.h"
#include "lib_echidna/delay.h"
#include "acedelay_parameters.h"

class AceDelayEffect : public EVEREST::IPlugin
{
public:
  virtual void SetSampleRate(float fSampleRate);

  // Audio processing.
  virtual void ProcessAudio(float** pInputs, float** pOutputs, long iNumSamples);

  // MIDI events.
  virtual void OnTempoChange(float fTempo);

  // meta-data
  virtual void GetId(char* pId) const
  {
    pId[0] = '@';
    pId[1] = '%';
    pId[2] = '0';
    pId[3] = 'v';
  }

  virtual int GetNumParams() const;
  virtual std::string GetParamName(int iParam) const;
  virtual float GetParam(int iIndex) const;
  virtual std::string GetParamText(int iIndex) const;
  virtual void SetParam(int iIndex, float fValue);

  AceDelayEffect();

private:
  float m_fTempo;
  float m_fSampleRate;

  Delay m_Delay[2];
  DelayParamSet m_Values;
};

