/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#include <string>
#include <vector>
#include <map>
#include <cmath>
#include "lib_everest/lib_everest.h"

using namespace std;

struct dsp
{
  float fSamplingFreq;
};

struct Meta
{
  void declare(const char* name, const char* value)
  {
    values[name] = value;
  }

  map<string, string> values;
};

struct UI
{
  void openVerticalBox(const char*)
  {
  }

  void closeBox()
  {
  }

  void declare(float* p, const char*, const char* name)
  {
    Param variable;
    variable.p = p;
    variable.name = name;
    params.push_back(variable);
  }

  void addHorizontalSlider(const char*, float*, float, float, float, float)
  {
  }

  struct Param
  {
    string name;
    float* p;
  };

  vector<Param> params;
};

#include "disto.dsp.h"

class EverestPlugin : public EVEREST::IPlugin
{
public:
  EverestPlugin()
  {
    m_dsp[0].metadata(&m_meta);

    m_dsp[0].buildUserInterface(&m_ui[0]);
    m_dsp[1].buildUserInterface(&m_ui[1]);
  }

  // creation
  void SetSampleRate(float fSampleRate)
  {
    m_dsp[0].instanceInit(fSampleRate);
    m_dsp[1].instanceInit(fSampleRate);
  }

  void ProcessAudio(float** pInput, float** pOutput, long iNumSamples)
  {
    float* leftInput[] =
    {
      pInput[0]
    };
    float* rightInput[] =
    {
      pInput[1]
    };

    float* leftOutput[] =
    {
      pOutput[0]
    };
    float* rightOutput[] =
    {
      pOutput[1]
    };

    m_dsp[0].compute(iNumSamples, leftInput, leftOutput);
    m_dsp[1].compute(iNumSamples, rightInput, rightOutput);
  }

  // midi events
  void OnTempoChange(float fTempo)
  {
  }

  void GetId(char* pId) const
  {
    pId[0] = 'D';
    pId[1] = 'I';
    pId[2] = 'S';
    pId[3] = 'T';
  }

  virtual int GetNumParams() const
  {
    return (int)m_ui[0].params.size();
  }

  float GetParam(int idx) const
  {
    return *m_ui[0].params[idx].p;
  }

  void SetParam(int idx, float fValue)
  {
    *m_ui[0].params[idx].p = fValue;
    *m_ui[1].params[idx].p = fValue;
  }

  string GetParamName(int idx) const
  {
    return m_ui[0].params[idx].name;
  }

  UI m_ui[2];
  mydsp m_dsp[2];
  Meta m_meta;
};

static EVEREST::IPlugin* CreatePlugin(EVEREST::IHost*)
{
  return new EverestPlugin;
}

namespace EVEREST
{
const PluginDescription g_PlugDesc =
{
  false, // isSynth
  { 0, 0 },
  "Ace",
  "AceDisto",
  &CreatePlugin,
  nullptr,
};
}

