/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#include "lib_utils/fraction.h"
#include "lib_everest/lib_everest.h"
#include "lib_echidna/filter.h"
#include "lib_echidna/moogfilter.h"
#include "acefilter_parameters.h"

class AceFilterEffect : public EVEREST::IPlugin
{
public:
  static EVEREST::IPlugin* CreateInstance(EVEREST::IHost* pHost);

  virtual void SetSampleRate(float fSampleRate);

  // Audio processing.
  virtual void ProcessAudio(float** pInputs, float** pOutputs, long iNumSamples);

  // MIDI events.
  virtual void OnTempoChange(float /*fTempo*/) {}

  virtual void GetId(char* pId) const
  {
    pId[0] = '@';
    pId[1] = '9';
    pId[2] = '#';
    pId[3] = '[';
  }

  virtual int GetNumParams() const;
  virtual std::string GetParamName(int iParam) const;
  virtual float GetParam(int iIndex) const;
  virtual std::string GetParamText(int iIndex) const;
  virtual void SetParam(int iIndex, float fValue);

private:
  AceFilterEffect(EVEREST::IHost* pHost);

  float m_fSampleRate;

  Lowpass m_Filter[2];
  Highpass m_HighPassFilter[2];
  MoogFilter m_MoogFilter[2];
  FilterParamSet m_Values;
  EVEREST::IHost* const m_pHost;
};

