/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#pragma once

#include "lib_utils/plugin_parameters.h"

enum FilterType
{
  FilterTypeLowpass,
  FilterTypeMoog,
  FilterTypeHighpass,
};

struct FilterParamSet
{
  FilterType filterType;
  float filterFreq;
  float filterReso;
};

FilterType const filterTypes[] =
{
  FilterTypeLowpass,
  FilterTypeMoog,
  FilterTypeHighpass,
};

static std::string internalToString(FilterType eType)
{
  switch(eType)
  {
  case FilterTypeLowpass: return "Lowpass";
  case FilterTypeHighpass: return "Highpass";
  case FilterTypeMoog: return "Moog";
  default: return "<?>";
  }
}

typedef ParamDesc<FilterParamSet, float> FloatParamDesc;
typedef EnumParamDesc<FilterParamSet, FilterType> FilterTypeParamDesc;

auto const DescType = FilterTypeParamDesc("type", filterTypes, &FilterParamSet::filterType);
auto const DescFreq = FloatParamDesc("freq", 0.00f, 100.0f, &FilterParamSet::filterFreq);
auto const DescReso = FloatParamDesc("reso", 0.00f, 100.0f, &FilterParamSet::filterReso);

