// SF2 based sampler

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#pragma once

#include <string>
#include <vector>
#include "lib_everest/lib_everest.h"

struct tsf;

class AceFontPlugin : public EVEREST::IPlugin,
  public EVEREST::IMidiListener
{
public:
  AceFontPlugin(EVEREST::IHost* pHost);
  ~AceFontPlugin();

  // EVEREST::IPlugin implementation
  virtual void SetSampleRate(float fSampleRate);

  virtual void ProcessAudio(float** pInput, float** pOutput, long iNumSamples);

  // midi events
  virtual void OnTempoChange(float fTempo);

  // IMidiListener
  virtual void noteOn(int iChannel, int iNote, int iVelocity);
  virtual void noteOff(int iChannel, int iNote, int iVelocity);
  virtual void pitchBend(int iChannel, float fBend);
  virtual void controlChange(int iChannel, int iControl, int iValue);

  virtual void GetId(char* pId) const
  {
    pId[0] = '@';
    pId[1] = '#';
    pId[2] = 'q';
    pId[3] = '!';
  }

  // parameters
  virtual int GetNumParams() const;
  virtual std::string GetParamName(int iParam) const;
  virtual std::string GetParamText(int iIndex) const;
  virtual float GetParam(int iIndex) const;
  virtual void SetParam(int iIndex, float fValue);

private:
  std::vector<float> m_fVstValues; // saved VST values (between 0 and 1). Only used when calling GetParam()
  tsf* m_sampler;
};

