// Trance gate effect plugin.
//
/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#include <algorithm>
#include <cassert>
#include <cmath>

#include "lib_utils/math_tools.h"

#include "acegate.h"

AceGateEffect::AceGateEffect()
{
  m_fTempo = 145.0;
  m_fSampleRate = 44100.0;
}

void AceGateEffect::ProcessAudio(float** pInputs, float** pOutputs, long iNumSamples)
{
  for(long i = 0; i < iNumSamples; ++i)
  {
    m_Gates[0].processSample(&pInputs[0][i], &pOutputs[0][i]);
    m_Gates[1].processSample(&pInputs[1][i], &pOutputs[1][i]);
  }
}

void AceGateEffect::noteOn(int /*channel*/, int note, int /*vel*/)
{
  auto const iDivisions = std::max(36, std::min(84, note)) - 36;
  auto const iNumSamplesNum = int(m_fSampleRate * 60.0 * iDivisions);
  auto const iNumSamplesDenom = int(m_fTempo * 32.0);
  auto const periodInSamples = Fraction(iNumSamplesNum, iNumSamplesDenom);
  m_Gates[0].On(periodInSamples);
  m_Gates[1].On(periodInSamples);
}

void AceGateEffect::noteOff(int /*channel*/, int /*note*/, int /*vel*/)
{
  m_Gates[0].Off();
  m_Gates[1].Off();
}

void AceGateEffect::SetSampleRate(float fSampleRate)
{
  m_fSampleRate = fSampleRate;
}

void AceGateEffect::OnTempoChange(float fTempo)
{
  m_fTempo = fTempo;
}

static EVEREST::IPlugin* CreatePlugin(EVEREST::IHost*)
{
  return new AceGateEffect;
}

namespace EVEREST
{
const PluginDescription g_PlugDesc =
{
  false, // isSynth
  { 0, 0 },
  "Ace",
  "AceGate",
  &CreatePlugin,
  nullptr,
};
}

