/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#include <algorithm>
#include "monogate.h"

MonoGate::MonoGate()
{
  m_bPlaying = false;
}

void MonoGate::On(Fraction const& periodInSamples)
{
  m_iPeriodInSamples = periodInSamples;
  m_bPlaying = true;
  m_iPosition = 0;
}

void MonoGate::Off()
{
  m_bPlaying = false;
}

void MonoGate::processSample(float const* in, float* out)
{
  // if not playing, pass-through.
  if(!m_bPlaying)
  {
    *out = *in;
    return;
  }

  // we are playing.
  auto const phase = (m_iPosition * m_iPeriodInSamples.denom) % (m_iPeriodInSamples.num);
  auto const half = m_iPeriodInSamples.num / 2;
  auto const ramp = 10 * m_iPeriodInSamples.denom;

  if(phase < half)
  {
    *out = *in;

    if(phase < ramp)
      *out *= float(phase) / float(ramp);
  }
  else if(phase < half + ramp)
  {
    *out = *in * float(phase - half) / float(ramp);
  }
  else
    *out = 0.0f;

  m_iPosition++;// = (m_iPosition + 1) % m_iPeriodInSamples;
}

