// Atomizer-like plugin.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */

#include <algorithm>
#include <stdio.h>
#include <cassert>
#include <cmath>
#include "acetrigger.h"

AceTriggerEffect::AceTriggerEffect()
{
  m_fTempo = 120.0;
  m_fSampleRate = 44100.0;
}

void AceTriggerEffect::ProcessAudio(float** pInputs, float** pOutputs, long iNumSamples)
{
  for(long i = 0; i < iNumSamples; ++i)
  {
    m_Triggers[0].processSample(&pInputs[0][i], &pOutputs[0][i]);
    m_Triggers[1].processSample(&pInputs[1][i], &pOutputs[1][i]);
  }
}

void AceTriggerEffect::noteOn(int /*channel*/, int iNote, int /*vel*/)
{
  int constrained_note = std::max(36, std::min(84, iNote)) - 36;
  int iNumSamples = int(m_fSampleRate * (60.0 / (m_fTempo * 32.0)) * float(constrained_note)) + 2;
  m_Triggers[0].On(iNumSamples);
  m_Triggers[1].On(iNumSamples);
}

void AceTriggerEffect::noteOff(int /*channel*/, int /*note*/, int /*vel*/)
{
  m_Triggers[0].Off();
  m_Triggers[1].Off();
}

void AceTriggerEffect::SetSampleRate(float fSampleRate)
{
  m_fSampleRate = fSampleRate;
}

void AceTriggerEffect::OnTempoChange(float fTempo)
{
  m_fTempo = fTempo;
}

static EVEREST::IPlugin* CreatePlugin(EVEREST::IHost*)
{
  return new AceTriggerEffect;
}

namespace EVEREST
{
const PluginDescription g_PlugDesc =
{
  false, // isSynth
  { 0, 0 },
  "Ace",
  "AceTrigger",
  &CreatePlugin,
  nullptr,
};
}

