/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#include <algorithm>
#include "monotrigger.h"

MonoTrigger::MonoTrigger()
{
  m_iNumSamplesInLoop = 6;
  m_bPlaying = false;

  // clear record buffer.
  m_iRecordPosition = 0;

  for(auto& val : m_fRecordBuffer)
    val = 0;
}

void MonoTrigger::On(int iDivisions)
{
  m_iNumSamplesInLoop = std::min<int>(iDivisions, MAX_SAMPLES_PER_CHUNK);
  m_bPlaying = true;

  m_iRecordPosition = 0;
  m_iPlayPosition = 0;
}

void MonoTrigger::Off()
{
  m_bPlaying = false;
}

void MonoTrigger::processSample(float* in, float* out)
{
  // if not playing, pass-through.
  if(!m_bPlaying)
  {
    *out = *in;
    return;
  }

  // we are playing.

  // if the record buffer is not full, add the current sample to it.
  if(m_iRecordPosition < m_iNumSamplesInLoop)
  {
    m_fRecordBuffer[m_iRecordPosition] = *in;
    ++m_iRecordPosition;
  }

  *out = m_fRecordBuffer[m_iPlayPosition];
  m_iPlayPosition = (m_iPlayPosition + 1) % m_iNumSamplesInLoop;
}

