/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#pragma once

#include "lib_utils/fraction.h"
#include "lib_everest/lib_everest.h"
#include "lib_echidna/filter.h"
#include "lib_echidna/biquad.h"
#include "lib_echidna/moogfilter.h"
#include "lib_compiler/compiled_program.h"
#include "aceversatile_parameters.h"
#include <mutex>
#include <memory>

class AceVersatileEffect : public EVEREST::IPlugin,
  public EVEREST::ISerializable
{
public:
  static EVEREST::IPlugin* CreateInstance(EVEREST::IHost* pHost);

  virtual void SetSampleRate(float fSampleRate);

  // Audio processing.
  virtual void ProcessAudio(float** pInputs, float** pOutputs, long iNumSamples);

  // MIDI events.
  virtual void OnTempoChange(float fTempo) {}

  virtual void GetId(char* pId) const
  {
    pId[0] = '!';
    pId[1] = '[';
    pId[2] = '#';
    pId[3] = '3';
  }

  // ISerializable
  std::vector<uint8_t> saveState() const override;
  void loadState(std::vector<uint8_t> const& data) override;

  virtual int GetNumParams() const;
  virtual std::string GetParamName(int iParam) const;
  virtual float GetParam(int iIndex) const;
  virtual std::string GetParamText(int iIndex) const;
  virtual void SetParam(int iIndex, float fValue);

  virtual void setProgramSource(std::string text);
  virtual std::string getText() const;

private:
  AceVersatileEffect(EVEREST::IHost* pHost);

  float m_fSampleRate;

  std::mutex m_protectProgram;
  std::unique_ptr<CompiledProgram> m_program[2];

  FilterParamSet m_Values;
  EVEREST::IHost* const m_pHost;
  std::string m_sText = "// type your program here";
};

