/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 */
#pragma once

#include "lib_utils/plugin_parameters.h"

#define ENABLE_FREE_BIQUAD 0

enum FilterType
{
  FilterTypeLowpass,
  FilterTypeMoog,
  FilterTypeFreeBiquad,
  FilterTypeHighpass,
};

struct FilterParamSet
{
  FilterType filterType;
  float filterFreq;
  float filterReso;
#if ENABLE_FREE_BIQUAD
  float a1, a2;
  float b0, b1, b2;
#endif
};

FilterType const filterTypes[] =
{
  FilterTypeLowpass,
  FilterTypeMoog,
#if ENABLE_FREE_BIQUAD
  FilterTypeFreeBiquad,
#endif
  FilterTypeHighpass,
};

static std::string internalToString(FilterType eType)
{
  switch(eType)
  {
  case FilterTypeLowpass: return "Lowpass";
  case FilterTypeHighpass: return "Highpass";
  case FilterTypeMoog: return "Moog";
  case FilterTypeFreeBiquad: return "FreeBiquad";
  default: return "<?>";
  }
}

typedef ParamDesc<FilterParamSet, float> FloatParamDesc;
typedef EnumParamDesc<FilterParamSet, FilterType> FilterTypeParamDesc;

auto const DescType = FilterTypeParamDesc("type", filterTypes, &FilterParamSet::filterType);
auto const DescFreq = FloatParamDesc("freq", 0.00f, 100.0f, &FilterParamSet::filterFreq);
auto const DescReso = FloatParamDesc("reso", 0.00f, 100.0f, &FilterParamSet::filterReso);

#if ENABLE_FREE_BIQUAD
auto const DescA1 = FloatParamDesc("a1", -1.00f, 1.0f, &FilterParamSet::a1);
auto const DescA2 = FloatParamDesc("a2", -1.00f, 1.0f, &FilterParamSet::a2);
auto const DescB0 = FloatParamDesc("b0", -1.00f, 1.0f, &FilterParamSet::b0);
auto const DescB1 = FloatParamDesc("b1", -1.00f, 1.0f, &FilterParamSet::b1);
auto const DescB2 = FloatParamDesc("b2", -1.00f, 1.0f, &FilterParamSet::b2);
#endif

