// Declaration of the MIDI-only Virus VST plugin.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#pragma once

#include "lib_everest/lib_everest.h"
#include "lib_rtmidi/RtMidi.h"
#include "lib_utils/midivirus.h"

#include <mutex>
#include <vector>
#include <string>
using namespace std;

#include "virus_params.h"

class AceVirusTi : public EVEREST::IPlugin,
  public EVEREST::ISerializable,
  public EVEREST::IMidiListener,
  public IMidiSink,
  public IMidiSource
{
public:
  AceVirusTi(EVEREST::IHost* pHost);

  // implementation of IMidiSink
  virtual void WriteMidi(uint8_t const* pData, int iLength);
  virtual void OnPatchReceived(int iPart, VirusPatch const& p);
  virtual void OnVirusParamChanged(int iPart, int iOffset, int iValue);
  virtual void Flush();
  virtual bool PullSysex(vector<uint8_t>& sysex_from_virus);

  // EVEREST::IPlugin implementation
  virtual void SetSampleRate(float fSampleRate);

  virtual void ProcessAudio(float** pInput, float** pOutput, long iNumSamples);

  // ISerializable
  vector<uint8_t> saveState() const override;
  void loadState(vector<uint8_t> const& data) override;

  virtual void OnTempoChange(float fTempo);

  // IMidiListener
  virtual void noteOn(int iChannel, int iNote, int iVelocity);
  virtual void noteOff(int iChannel, int iNote, int iVelocity);
  virtual void pitchBend(int iChannel, float fBend);
  virtual void controlChange(int, int, int) {};

  virtual void GetId(char* pId) const
  {
    pId[0] = '@';
    pId[1] = '$';
    pId[2] = '{';
    pId[3] = ']';
  }

  // params
  virtual int GetNumParams() const;
  virtual string GetParamName(int iParam) const;
  virtual float GetParam(int iIndex) const;
  virtual string GetParamText(int iIndex) const;
  virtual void SetParam(int iIndex, float fValue);

  // called by the editor
  vector<string> GetInputDeviceList() const;
  vector<string> GetOutputDeviceList() const;
  void ChangeMidiOutputDevice(string const& sName);
  void ChangeMidiInputDevice(string const& sName);
  bool IsInputOK() const;
  bool IsOutputOK() const;
  bool DownloadPatch();
  bool UploadPatch();

  void SetPatch(VirusPatch const& p)
  {
    std::lock_guard<std::recursive_mutex> lock_it(m_PatchLock);
    m_Patch = p;
  }

  VirusPatch GetPatch() const
  {
    std::lock_guard<std::recursive_mutex> lock_it(m_PatchLock);
    return m_Patch;
  }

  string GetMidiInputDeviceName() const
  {
    return m_sInputDeviceName;
  }

  string GetMidiOutputDeviceName() const
  {
    return m_sOutputDeviceName;
  }

  int GetPart() const
  {
    return m_PartNumber;
  }

  void SetPart(int iPart)
  {
    m_PartNumber = iPart;
  }

  void InternalSetParam(int iIndex, float fValue, bool bSendMidi, bool bNotifyHost);

private:
  float m_fValues[128 * 4];
  EVEREST::IHost* const m_pHost;
  mutable std::recursive_mutex m_PatchLock;
  RtMidiIn m_MidiInput;
  string m_sInputDeviceName, m_sOutputDeviceName;
  RtMidiOut m_MidiOutput;
  VirusPatch m_Patch;
  int m_PartNumber; // 0-based

  CMidiVirus m_MidiVirus;
  int GetDeviceId(bool bInput = true);
  void ReconnectMidi();

  void midiInputCallback(double timeStamp, vector<uint8_t> const& message);

  static void static_callback(double timeStamp, vector<uint8_t>* message, void* userData)
  {
    AceVirusTi* pThis = reinterpret_cast<AceVirusTi*>(userData);
    pThis->midiInputCallback(timeStamp, *message);
  }
};

