// Definition of the GUI part of the plugin.

/*
 * Copyright (C) 2017 - Sebastien Alaiwan
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#include <sstream>
#include <cassert>
#include "acevirusti_editor.h"
#include "acevirusti.h"
#include "lib_utils/math_tools.h"
#include "../revision.h"
#include "lib_everest/gui_knob.h"

AceVirusTiEditor::AceVirusTiEditor(AceVirusTi* pPlugin) :
  m_pVirusPlugin(pPlugin)
{
  m_iPartNumber = 0;

  setBounds(0, 0, WIDTH, HEIGHT);
  // setSize(WIDTH, HEIGHT);

  int line = 10;

  int col = 10;

  m_pEditPatchName = new juce::TextEditor;
  m_pEditPatchName->setReadOnly(true);
  m_pEditPatchName->setBounds(col, line, 90, 20);
  addAndMakeVisible(m_pEditPatchName);
  col += 100;

  m_pButtonRecv = new juce::TextButton;
  m_pButtonRecv->setButtonText("Recv");
  m_pButtonRecv->setBounds(col, line, 50, 20);
  AddButton(m_pButtonRecv);
  col += 60;

  m_pButtonSend = new juce::TextButton;
  m_pButtonSend->setTopLeftPosition(10, 10);
  m_pButtonSend->setButtonText("Send");
  m_pButtonSend->setBounds(col, line, 50, 20);
  AddButton(m_pButtonSend);
  col += 60;

  m_pButtonExport = new juce::TextButton;
  m_pButtonExport->setTopLeftPosition(10, 10);
  m_pButtonExport->setButtonText("Export");
  m_pButtonExport->setBounds(col, line, 50, 20);
  AddButton(m_pButtonExport);
  col += 60;

  m_pButtonImport = new juce::TextButton;
  m_pButtonImport->setTopLeftPosition(10, 10);
  m_pButtonImport->setButtonText("Import");
  m_pButtonImport->setBounds(col, line, 50, 20);
  AddButton(m_pButtonImport);
  col += 60;

  line += 24;

  {
    juce::DrawableText* pText = new juce::DrawableText;
    pText->setBoundingBox
    (
      RelativeParallelogram
      (
        RelativePoint(10.0f, (float)line),
        RelativePoint(115.0f, (float)line),
        RelativePoint(10.0f, (float)line + 25.0f)
      )
    );
    pText->setFont(Font(16.0f), true);
    pText->setText("Virus MIDI input:");
    pText->setColour(juce::Colour(0xFFFFFFFF));
    addAndMakeVisible(pText);

    std::vector<std::string> input_devices = m_pVirusPlugin->GetInputDeviceList();

    juce::ComboBox* pCombo = new juce::ComboBox;

    for(size_t i = 0; i < input_devices.size(); ++i)
    {
      pCombo->addItem(juce::String::fromUTF8(input_devices[i].c_str()), i + 1);
    }

    pCombo->setTopLeftPosition(10, 10);
    pCombo->setBounds(120, line, 200, 20);
    pCombo->addListener(this);
    addAndMakeVisible(pCombo);
    m_pCombo[0] = pCombo;
  }

  line += 24;

  {
    juce::DrawableText* pText = new juce::DrawableText;
    pText->setBoundingBox
    (
      RelativeParallelogram
      (
        RelativePoint(10.0f, (float)line),
        RelativePoint(115.0f, (float)line),
        RelativePoint(10.0f, (float)line + 25.0f)
      )
    );
    pText->setFont(Font(16.0f), true);
    pText->setText("Virus MIDI output:");
    pText->setColour(juce::Colour(0xFFFFFFFF));
    addAndMakeVisible(pText);

    std::vector<std::string> devices = m_pVirusPlugin->GetOutputDeviceList();

    juce::ComboBox* pCombo = new juce::ComboBox;

    for(size_t i = 0; i < devices.size(); ++i)
    {
      pCombo->addItem(juce::String::fromUTF8(devices[i].c_str()), i + 1);
    }

    pCombo->setTopLeftPosition(10, 10);
    pCombo->setBounds(120, line, 200, 20);
    pCombo->addListener(this);
    addAndMakeVisible(pCombo);
    m_pCombo[1] = pCombo;
  }

  line += 24;

  {
    juce::DrawableText* pText = new juce::DrawableText;
    pText->setBoundingBox
    (
      RelativeParallelogram
      (
        RelativePoint(10.0f, (float)line),
        RelativePoint(115.0f, (float)line),
        RelativePoint(10.0f, (float)line + 25.0f)
      )
    );
    pText->setFont(Font(16.0f), true);
    pText->setText("Virus Part:");
    pText->setColour(juce::Colour(0xFFFFFFFF));
    addAndMakeVisible(pText);

    juce::ComboBox* pCombo = new juce::ComboBox;

    for(int i = 0; i < 16; ++i)
    {
      char text[32];
      sprintf(text, "Part %.2d", i + 1);
      pCombo->addItem(juce::String::fromUTF8(text), i + 1);
    }

    pCombo->setTopLeftPosition(10, 10);
    pCombo->setBounds(120, line, 100, 20);
    pCombo->addListener(this);
    addAndMakeVisible(pCombo);
    m_pCombo[2] = pCombo;
  }

  {
    juce::DrawableText* pText = new juce::DrawableText;
    pText->setBoundingBox
    (
      RelativeParallelogram
      (
        RelativePoint(260 + 10.0f, (float)line),
        RelativePoint(260 + 150.0f, (float)line),
        RelativePoint(260 + 10.0f, (float)line + 25.0f)
      )
    );
    pText->setFont(Font(16.0f), true);
#define QUOTEME_(x) # x
#define QUOTEME(x) QUOTEME_(x)
    pText->setText("AceVirusTi r" QUOTEME(REVISION));
    pText->setColour(juce::Colour(0xFFFFFFFF));
    addAndMakeVisible(pText);
  }

  line += 16;

  // add the tabs
  AddTabs();

  ProtectedRefresh();
  startTimer(100);
}

void AceVirusTiEditor::timerCallback()
{
  VirusPatch p = m_pVirusPlugin->GetPatch();
  m_pEditPatchName->setText(p.GetName().c_str());

  // update knob positions
  for(size_t i = 0; i < m_Knobs.size(); ++i)
    m_KnobListeners[i]->SyncToPlugin(m_Knobs[i]);
}

void AceVirusTiEditor::AddTabs()
{
  auto tabs = new juce::TabbedComponent(juce::TabbedButtonBar::TabsAtBottom);
  addAndMakeVisible(tabs);
  tabs->setTopLeftPosition(10, 120);
  tabs->setSize(WIDTH - 2 * 10, 470);

  juce::Component* currTab = nullptr;
  int numTabs = 0;

  auto const tabColour = juce::Colour(64, 64, 64);

  // add the knobs
  {
    auto const KNOBS_PER_TAB = 64;

    int const x0 = 10;
    int const y0 = 10;
    int const x_step = 96;
    int const y_step = 26;

    int x = x0;
    int y = y0;
    int knobsInThisTab = 0;

    for(int i = 0; i < m_pVirusPlugin->GetNumParams(); ++i)
    {
      if(knobsInThisTab % 4 == 0 && knobsInThisTab > 0)
      {
        x = x0;
        y += y_step;
      }

      if(currTab == nullptr)
      {
        currTab = new juce::Component;
        ++numTabs;
        std::stringstream ssTabName;
        ssTabName << "Page " << numTabs;
        tabs->addTab(ssTabName.str().c_str(), tabColour, currTab, true);
        knobsInThisTab = 0;
        x = x0;
        y = y0;
      }

      auto pListener = new KnobParamMapper(i, m_pVirusPlugin);

      auto pKnob = new EVEREST::KnobComponent(pListener);
      pKnob->setTopLeftPosition(x, y);
      currTab->addAndMakeVisible(pKnob);

      {
        int xx = x + 18;
        int yy = y;
        int w = (x_step * 2) / 3 + 2;
        auto pText = new juce::DrawableText;
        auto box = RelativeParallelogram
                   (
          RelativePoint(xx, yy),
          RelativePoint(xx + w, yy),
          RelativePoint(xx, yy + 10)
                   );
        pText->setBoundingBox(box);
        pText->setFont(Font(10.0f), true);
        pText->setText(m_pVirusPlugin->GetParamName(i).c_str());
        pText->setColour(juce::Colour(0xFFFFFFFF));
        currTab->addAndMakeVisible(pText);
      }

      knobsInThisTab++;

      if(knobsInThisTab == KNOBS_PER_TAB)
        currTab = nullptr;

      m_Knobs.push_back(pKnob);
      m_KnobListeners.push_back(std::unique_ptr<KnobParamMapper>(pListener));

      x += x_step;
    }
  }

  tabs->setTabBarDepth(30);
  tabs->setIndent(5);
}

void AceVirusTiEditor::buttonClicked(juce::Button* button)
{
  if(button == m_pButtonRecv)
    SavePatch();
  else if(button == m_pButtonSend)
    LoadPatch();
  else if(button == m_pButtonExport)
    ExportPatch();
  else if(button == m_pButtonImport)
    ImportPatch();
  else
  {
    // assert(0);
  }
}

void AceVirusTiEditor::comboBoxChanged(juce::ComboBox* cb)
{
  int idx = cb->getSelectedItemIndex();
  std::string s(cb->getItemText(idx).getCharPointer());

  if(cb == m_pCombo[0])
  {
    m_pVirusPlugin->ChangeMidiInputDevice(s);
    ProtectedRefresh();
  }
  else if(cb == m_pCombo[1])
  {
    m_pVirusPlugin->ChangeMidiOutputDevice(s);
    ProtectedRefresh();
  }
  else if(cb == m_pCombo[2])
  {
    m_pVirusPlugin->SetPart(idx);
    ProtectedRefresh();
  }
}

void AceVirusTiEditor::AddButton(juce::Button* pButton)
{
  pButton->addListener(this);
  addAndMakeVisible(pButton);
}

AceVirusTiEditor::~AceVirusTiEditor()
{
  this->deleteAllChildren();
}

void AceVirusTiEditor::paint(juce::Graphics& g)
{
  g.fillAll(juce::Colour(0xff101010));
}

static std::string DoFileDialog()
{
  juce::FileChooser myChooser("Choose your patch",
                              juce::File::getSpecialLocation(juce::File::userHomeDirectory),
                              "*.viruspatch");

  if(!myChooser.browseForFileToOpen())
    return "";

  File patchFile = myChooser.getResult();
  auto path = patchFile.getFullPathName();
  char const* p = path.toUTF8();
  return p;
}

void AceVirusTiEditor::SavePatch()
{
  m_pVirusPlugin->DownloadPatch();
  ProtectedRefresh();
}

void AceVirusTiEditor::LoadPatch()
{
  m_pVirusPlugin->UploadPatch();
}

void AceVirusTiEditor::ExportPatch()
{
  bool bIsMidi = false;
  std::string sPath = DoFileDialog();

  if(sPath.empty())
    return;

  VirusPatch p = m_pVirusPlugin->GetPatch();

  if(bIsMidi)
  {
    if(!p.ExportMidiFile(sPath))
    {
      AlertWindow::showMessageBox(AlertWindow::WarningIcon, T("Error"), T("Failed saving file."));
      return;
    }
  }
  else
  {
    if(!p.Export(sPath))
    {
      AlertWindow::showMessageBox(AlertWindow::WarningIcon, T("Error"), T("Failed saving file."));
      return;
    }
  }
}

void AceVirusTiEditor::ImportPatch()
{
  std::string sPath = DoFileDialog();

  if(sPath.empty())
    return;

  VirusPatch p;

  if(!p.Import(sPath))
  {
    AlertWindow::showMessageBox(AlertWindow::WarningIcon, T("Error"), T("Failed importing file."));
    return;
  }

  m_pVirusPlugin->SetPatch(p);

  ProtectedRefresh();
}

static void ComboSelectItemByText(juce::ComboBox* p, std::string const& sText)
{
  int const n = p->getNumItems();

  for(int i = 0; i < n; ++i)
  {
    std::string s(p->getItemText(i).getCharPointer());

    if(s == sText)
    {
      p->setSelectedItemIndex(i);
      return;
    }
  }
}

void AceVirusTiEditor::ProtectedRefresh()
{
  // without input, we can still send patches to the virus
  m_pButtonSend->setEnabled(m_pVirusPlugin->IsOutputOK());

  // without output, we can't communicate with the virus.
  m_pButtonRecv->setEnabled(m_pVirusPlugin->IsInputOK() && m_pVirusPlugin->IsOutputOK());

  ComboSelectItemByText(m_pCombo[0], m_pVirusPlugin->GetMidiInputDeviceName());
  ComboSelectItemByText(m_pCombo[1], m_pVirusPlugin->GetMidiOutputDeviceName());

  m_pCombo[2]->setSelectedItemIndex(m_pVirusPlugin->GetPart());
}

