/**
 * @file virus_editor.h
 * @brief Declaration of the GUI part of the plugin.
 * @author Sebastien Alaiwan
 * @version 1.0
 * @date 2010-09-15
 */

#pragma once

#include <map>
#include <string>
#include "acevirusti.h"
#include "lib_juce/juce.h"
#include "lib_utils/midivirus.h"
#include <memory>
#include "lib_everest/gui_knob.h"

class KnobParamMapper : public EVEREST::KnobListener
{
public:
  int const m_iParamIndex;
  KnobParamMapper(int iParamIndex, AceVirusTi* pVirusPlugin) : m_iParamIndex(iParamIndex), m_pPlugin(pVirusPlugin)
  {
  }

  void OnValueChanged(EVEREST::KnobComponent* pKnob)
  {
    float fPos = pKnob->GetPosition();
    m_pPlugin->InternalSetParam(m_iParamIndex, fPos, true, true);
  }

  void SyncToPlugin(EVEREST::KnobComponent* pKnob)
  {
    if(pKnob->IsDragging())
      return;

    float fPos = m_pPlugin->GetParam(m_iParamIndex);
    pKnob->setPosition(fPos);
  }

private:
  AceVirusTi* const m_pPlugin;
};

class AceVirusTiEditor : public juce::Component, public juce::ButtonListener, public juce::ComboBoxListener, public juce::Timer
{
private:
  AceVirusTi* const m_pVirusPlugin;

  void AddButton(juce::Button*);
  void AddTabs();
  void OnPressButton(int id);
  void OnValueChanged(EVEREST::KnobComponent* pKnob);

  void SavePatch();
  void LoadPatch();
  void ExportPatch();
  void ImportPatch();

  int m_iPartNumber;

public:
  AceVirusTiEditor(AceVirusTi* pEffect);
  virtual ~AceVirusTiEditor();

  void buttonClicked(juce::Button* button);
  void comboBoxChanged(juce::ComboBox* cb);

  static const int WIDTH = 430;
  static const int HEIGHT = 590;

  void ProtectedRefresh();
  void paint(juce::Graphics& g);
  void timerCallback();

  juce::Button* m_pButtonRecv;
  juce::Button* m_pButtonSend;
  juce::Button* m_pButtonExport;
  juce::Button* m_pButtonImport;
  juce::TextEditor* m_pEditPatchName;
  juce::ComboBox* m_pCombo[3];

  std::vector<std::unique_ptr<KnobParamMapper>> m_KnobListeners;
  std::vector<EVEREST::KnobComponent*> m_Knobs;
};

