#!dmd -run
import std.conv;
import std.stdio;
import std.process;
import std.string;
import std.file;
import std.path;

int main(string[] args)
{
  writefln("Fetching current revision ...");
  string sRev = chomp(executeShell("bzr revno lp:acevirus").output);

  writefln("Current revision is : %s", sRev);

  int rev = to!int(sRev);
  string cwd = getcwd();
  string BASE_DIR = cwd;
  string prefix = std.string.format("OfficialBuild-%d", rev);

  string SRC_DIR   = buildPath(BASE_DIR, prefix ~ "-src");
  string BUILD_DIR = buildPath(BASE_DIR, prefix ~ "-build");
  string PROD_DIR  = buildPath(BASE_DIR, prefix ~ "-prod");

  writeln("PROD_DIR=", PROD_DIR);
  writeln("BUILD_DIR=", BUILD_DIR);

  mkdir(PROD_DIR);
  mkdir(BUILD_DIR);

  EnsureExec("bzr", "export", SRC_DIR, "lp:acevirus", "-r", to!string(rev));
  chdir(SRC_DIR);

  std.file.write("revision.h", std.string.format("#define REVISION %d", rev));

  // Linux 64 bit build
  {
    string BUILD  = buildPath(BUILD_DIR, "linux64");
    mkdir(BUILD);

    environment["PKG_CONFIG_LIBDIR"] = "/usr/lib/x86_64-linux-gnu/pkgconfig";
    environment["CROSS_COMPILE"] = "x86_64-linux-gnu-";
    environment["BIN"] = BUILD;
    environment["EXT"] = "so";
    environment["CXXFLAGS"] = "-fPIC";
    environment["LDFLAGS"] = "";

    EnsureExec("make", "-j", "8");

    void copyTaggedLinux(string filename)
    {
      string taggedName = std.string.format("%s-%d.so", baseName(filename, ".vst.so"), rev);
      copy(buildPath(BUILD, filename), buildPath(PROD_DIR, taggedName));
      //EnsureExec("upx", "-o", buildPath(PROD_DIR, taggedName), buildPath(BUILD, "bin", "Release", filename));
    }

    copyTaggedLinux("aceechidna.vst.so");
    copyTaggedLinux("acecrusher.vst.so");
    copyTaggedLinux("acetrigger.vst.so");
    copyTaggedLinux("acevirusti.vst.so");
    copyTaggedLinux("acegate.vst.so");
    copyTaggedLinux("acefilter.vst.so");
  }

  // Windows 32 bit build.
  {
    string BUILD = buildPath(BUILD_DIR, "w32");
    mkdir(BUILD);

    const libs = [
      "-limm32",
      "-lwinmm",
      "-lgdi32",
      "-luuid",
      "-lvfw32",
      "-lwininet",
      "-lwinmm",
      "-lopengl32",
      "-loleaut32",
      "-lws2_32",
      "-lole32",
      "-lcomdlg32",
      "-lversion",
      "-lshlwapi",
    ];

    environment["PKG_CONFIG_LIBDIR"] = "/opt/envs/i686-ace-mingw32/lib/pkgconfig";
    environment["CROSS_COMPILE"] = "i686-w64-mingw32-";
    environment["BIN"] = BUILD;
    environment["EXT"] = "DLL";
    environment["LDFLAGS"] = join(libs, " ") ~ " -static";

    EnsureExec("make", "-j", "8");

    void copyTagged(string filename)
    {
      string taggedName = std.string.format("%s-%d.dll", baseName(filename, ".vst.DLL"), rev);
      EnsureExec("upx", "-o", buildPath(PROD_DIR, taggedName), buildPath(BUILD, filename));
    }

    copyTagged("aceechidna.vst.DLL");
    copyTagged("acecrusher.vst.DLL");
    copyTagged("acetrigger.vst.DLL");
    copyTagged("acevirusti.vst.DLL");
    copyTagged("acegate.vst.DLL");
    copyTagged("acefilter.vst.DLL");
  }

  return 0;
}

void EnsureExec(string[] args...)
{
  writefln("Exec: %s", args);

  auto pid = spawnProcess(args);
  int ret = wait(pid);
  if(ret != 0)
  {
    writefln("Error : %d", ret);
    assert(false);
  }
}

