/**
 * @file virus_editor.h
 * @brief Declaration of the GUI part of the plugin.
 * @author Sebastien Alaiwan
 * @version 1.0
 * @date 2010-09-15
 */

#pragma once

#include <map>
#include <string>
#include "midivirus.h"
#include "juce.h"

class AceVirus;

class VirusEditor : public juce::Component, public juce::ButtonListener, public juce::ComboBoxListener
{
private:
  bool m_bPatchValid[16];

  AceVirus* const m_pVirusPlugin;

  void AddButton(juce::Button*, int iID);
  void OnPressButton(int id);

  void SavePatch(int iPatchNumber);
  void LoadPatch(int iPatchNumber);
  void ExportPatch(int iPatchNumber);
  void ImportPatch(int iPatchNumber);

public:
  VirusEditor(AceVirus* pEffect);
  virtual ~VirusEditor();

  void buttonClicked(juce::Button* button);
  void comboBoxChanged(juce::ComboBox* cb);

  static const int WIDTH = 400;
  static const int HEIGHT = 500;

  void ProtectedRefresh();
  void paint(juce::Graphics& g);

  std::map<juce::Button*, int> m_ButtonToId;
  std::map<int, juce::Button*> m_IdToButton;
  juce::TextEditor* m_pEdit[16];
  juce::ComboBox* m_pCombo[2];
};

